@@ -53,14 +53,8 @@ Cluster::Cluster(Server *svr, std::vector<std::string> binds, int port)
 // cluster data, so these commands should be executed exclusively, and ReadWriteLock
 // also can guarantee accessing data is safe.
 bool Cluster::SubCommandIsExecExclusive(const std::string &subcommand) {
-  if (strcasecmp("setnodes", subcommand.c_str()) == 0) {
-    return true;
-  } else if (strcasecmp("setnodeid", subcommand.c_str()) == 0) {
-    return true;
-  } else if (strcasecmp("setslot", subcommand.c_str()) == 0) {
-    return true;
-  } else if (strcasecmp("import", subcommand.c_str()) == 0) {
-    return true;
+  for (auto v : {"setnodes", "setnodeid", "setslot", "import"}) {
+    if (util::EqualICase(v, subcommand)) return true;
   }
   return false;
 }
@@ -133,7 +127,10 @@ Status Cluster::SetSlotRanges(const std::vector<SlotRange> &slot_ranges, const s
       if (old_node == myself_ && old_node != to_assign_node) {
         // If slot is migrated from this node
         if (migrated_slots_.count(slot) > 0) {
-          svr_->slot_migrator->ClearKeysOfSlot(kDefaultNamespace, slot);
+          auto s = svr_->slot_migrator->ClearKeysOfSlot(kDefaultNamespace, slot);
+          if (!s.ok()) {
+            LOG(ERROR) << "failed to clear data of migrated slot: " << s.ToString();
+          }
           migrated_slots_.erase(slot);
         }
         // If slot is imported into this node
@@ -215,7 +212,10 @@ Status Cluster::SetClusterNodes(const std::string &nodes_str, int64_t version, b
   if (!migrated_slots_.empty()) {
     for (auto &it : migrated_slots_) {
       if (slots_nodes_[it.first] != myself_) {
-        svr_->slot_migrator->ClearKeysOfSlot(kDefaultNamespace, it.first);
+        auto s = svr_->slot_migrator->ClearKeysOfSlot(kDefaultNamespace, it.first);
+        if (!s.ok()) {
+          LOG(ERROR) << "failed to clear data of migrated slots: " << s.ToString();
+        }
       }
     }
   }
@@ -667,9 +667,9 @@ Status Cluster::parseClusterNodes(const std::string &nodes_str, ClusterNodes *no
 
     // 4) role
     int role = 0;
-    if (strcasecmp(fields[3].c_str(), "master") == 0) {
+    if (util::EqualICase(fields[3], "master")) {
       role = kClusterMaster;
-    } else if (strcasecmp(fields[3].c_str(), "slave") == 0 || strcasecmp(fields[3].c_str(), "replica") == 0) {
+    } else if (util::EqualICase(fields[3], "slave") || util::EqualICase(fields[3], "replica")) {
       role = kClusterSlave;
     } else {
       return {Status::ClusterInvalidInfo, "Invalid cluster node role"};
@@ -748,7 +748,7 @@ bool Cluster::IsWriteForbiddenSlot(int slot) { return svr_->slot_migrator->GetFo
 Status Cluster::CanExecByMySelf(const redis::CommandAttributes *attributes, const std::vector<std::string> &cmd_tokens,
                                 redis::Connection *conn) {
   std::vector<int> keys_indexes;
-  auto s = redis::GetKeysFromCommand(attributes->name, static_cast<int>(cmd_tokens.size()), &keys_indexes);
+  auto s = redis::GetKeysFromCommand(attributes, cmd_tokens, &keys_indexes);
   // No keys
   if (!s.IsOK()) return Status::OK();
 
@@ -779,7 +779,7 @@ Status Cluster::CanExecByMySelf(const redis::CommandAttributes *attributes, cons
     }
     // To keep data consistency, slot will be forbidden write while sending the last incremental data.
     // During this phase, the requests of the migrating slot has to be rejected.
-    if (attributes->IsWrite() && IsWriteForbiddenSlot(slot)) {
+    if ((attributes->flags & redis::kCmdWrite) && IsWriteForbiddenSlot(slot)) {
       return {Status::RedisExecErr, "TRYAGAIN Can't write to slot being migrated which is in write forbidden phase"};
     }
 
@@ -801,7 +801,7 @@ Status Cluster::CanExecByMySelf(const redis::CommandAttributes *attributes, cons
     return Status::OK();  // I'm serving the imported slot
   }
 
-  if (myself_ && myself_->role == kClusterSlave && !attributes->IsWrite() &&
+  if (myself_ && myself_->role == kClusterSlave && !(attributes->flags & redis::kCmdWrite) &&
       nodes_.find(myself_->master_id) != nodes_.end() && nodes_[myself_->master_id] == slots_nodes_[slot]) {
     return Status::OK();  // My master is serving this slot
   }
