@@ -307,15 +307,13 @@ Status SlotMigrator::sendSnapshot() {
 
   LOG(INFO) << "[migrate] Start migrating snapshot of slot " << slot;
 
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = slot_snapshot_;
-  storage_->SetReadOptions(read_options);
   rocksdb::ColumnFamilyHandle *cf_handle = storage_->GetCFHandle(engine::kMetadataColumnFamilyName);
   auto iter = util::UniqueIterator(storage_->GetDB()->NewIterator(read_options, cf_handle));
 
   // Construct key prefix to iterate the keys belong to the target slot
-  std::string prefix;
-  ComposeSlotKeyPrefix(namespace_, slot, &prefix);
+  std::string prefix = ComposeSlotKeyPrefix(namespace_, slot);
   LOG(INFO) << "[migrate] Iterate keys of slot, key's prefix: " << prefix;
 
   // Seek to the beginning of keys start with 'prefix' and iterate all these keys
@@ -332,8 +330,7 @@ Status SlotMigrator::sendSnapshot() {
     }
 
     // Get user key
-    std::string ns, user_key;
-    ExtractNamespaceKey(iter->key(), &ns, &user_key, true);
+    auto [_, user_key] = ExtractNamespaceKey(iter->key(), true);
 
     // Add key's constructed commands to restore_cmds, send pipeline or not according to task's max_pipeline_size
     auto result = migrateOneKey(user_key, iter->value(), &restore_cmds);
@@ -364,7 +361,7 @@ Status SlotMigrator::sendSnapshot() {
   }
 
   LOG(INFO) << "[migrate] Succeed to migrate slot snapshot, slot: " << slot << ", Migrated keys: " << migrated_key_cnt
-            << ", Expired keys: " << expired_key_cnt << ", Emtpy keys: " << empty_key_cnt;
+            << ", Expired keys: " << expired_key_cnt << ", Empty keys: " << empty_key_cnt;
 
   return Status::OK();
 }
@@ -587,7 +584,9 @@ StatusOr<KeyMigrationResult> SlotMigrator::migrateOneKey(const rocksdb::Slice &k
                                                          std::string *restore_cmds) {
   std::string bytes = encoded_metadata.ToString();
   Metadata metadata(kRedisNone, false);
-  metadata.Decode(bytes);
+  if (auto s = metadata.Decode(bytes); !s.ok()) {
+    return {Status::NotOK, s.ToString()};
+  }
 
   if (metadata.Type() != kRedisString && metadata.Type() != kRedisStream && metadata.size == 0) {
     return KeyMigrationResult::kUnderlyingStructEmpty;
@@ -620,7 +619,9 @@ StatusOr<KeyMigrationResult> SlotMigrator::migrateOneKey(const rocksdb::Slice &k
     }
     case kRedisStream: {
       StreamMetadata stream_md(false);
-      stream_md.Decode(bytes);
+      if (auto s = stream_md.Decode(bytes); !s.ok()) {
+        return {Status::NotOK, s.ToString()};
+      }
 
       auto s = migrateStream(key, stream_md, restore_cmds);
       if (!s.IsOK()) {
@@ -660,16 +661,14 @@ Status SlotMigrator::migrateComplexKey(const rocksdb::Slice &key, const Metadata
   cmd = type_to_cmd[metadata.Type()];
 
   std::vector<std::string> user_cmd = {cmd, key.ToString()};
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = slot_snapshot_;
-  storage_->SetReadOptions(read_options);
   // Should use th raw db iterator to avoid reading uncommitted writes in transaction mode
   auto iter = util::UniqueIterator(storage_->GetDB()->NewIterator(read_options));
 
   // Construct key prefix to iterate values of the complex type user key
-  std::string slot_key, prefix_subkey;
-  AppendNamespacePrefix(key, &slot_key);
-  InternalKey(slot_key, "", metadata.version, true).Encode(&prefix_subkey);
+  std::string slot_key = AppendNamespacePrefix(key);
+  std::string prefix_subkey = InternalKey(slot_key, "", metadata.version, true).Encode();
   int item_count = 0;
 
   for (iter->Seek(prefix_subkey); iter->Valid(); iter->Next()) {
@@ -762,18 +761,15 @@ Status SlotMigrator::migrateComplexKey(const rocksdb::Slice &key, const Metadata
 }
 
 Status SlotMigrator::migrateStream(const Slice &key, const StreamMetadata &metadata, std::string *restore_cmds) {
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = slot_snapshot_;
-  storage_->SetReadOptions(read_options);
   // Should use th raw db iterator to avoid reading uncommitted writes in transaction mode
   auto iter = util::UniqueIterator(
       storage_->GetDB()->NewIterator(read_options, storage_->GetCFHandle(engine::kStreamColumnFamilyName)));
 
-  std::string ns_key;
-  AppendNamespacePrefix(key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(key);
   // Construct key prefix to iterate values of the stream
-  std::string prefix_key;
-  InternalKey(ns_key, "", metadata.version, true).Encode(&prefix_key);
+  std::string prefix_key = InternalKey(ns_key, "", metadata.version, true).Encode();
 
   std::vector<std::string> user_cmd = {type_to_cmd[metadata.Type()], key.ToString()};
 
