@@ -37,6 +37,7 @@ class CommandType : public Commander {
     RedisType type = kRedisNone;
     auto s = redis.Type(args_[1], &type);
     if (s.ok()) {
+      if (type >= RedisTypeNames.size()) return {Status::RedisExecErr, "Invalid type"};
       *output = redis::SimpleString(RedisTypeNames[type]);
       return Status::OK();
     }
@@ -45,6 +46,24 @@ class CommandType : public Commander {
   }
 };
 
+class CommandMove : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    GET_OR_RET(ParseInt<int64_t>(args[2], 10));
+    return Status::OK();
+  }
+
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    int count = 0;
+    redis::Database redis(svr->storage, conn->GetNamespace());
+    rocksdb::Status s = redis.Exists({args_[1]}, &count);
+    if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
+
+    *output = count ? redis::Integer(1) : redis::Integer(0);
+    return Status::OK();
+  }
+};
+
 class CommandObject : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
@@ -61,7 +80,7 @@ class CommandObject : public Commander {
         output->append(redis::BulkString(info));
       }
     } else {
-      *output = redis::Error("object subcommand must be dump");
+      return {Status::RedisExecErr, "object subcommand must be dump"};
     }
     return Status::OK();
   }
@@ -99,13 +118,15 @@ class CommandExists : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     std::vector<rocksdb::Slice> keys;
+    keys.reserve(args_.size() - 1);
     for (size_t i = 1; i < args_.size(); i++) {
       keys.emplace_back(args_[i]);
     }
 
     int cnt = 0;
     redis::Database redis(svr->storage, conn->GetNamespace());
-    redis.Exists(keys, &cnt);
+    auto s = redis.Exists(keys, &cnt);
+    if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
     *output = redis::Integer(cnt);
 
     return Status::OK();
@@ -236,12 +257,17 @@ class CommandPersist : public Commander {
 class CommandDel : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    int cnt = 0;
-    redis::Database redis(svr->storage, conn->GetNamespace());
+    std::vector<rocksdb::Slice> keys;
+    keys.reserve(args_.size() - 1);
     for (size_t i = 1; i < args_.size(); i++) {
-      auto s = redis.Del(args_[i]);
-      if (s.ok()) cnt++;
+      keys.emplace_back(args_[i]);
     }
+
+    uint64_t cnt = 0;
+    redis::Database redis(svr->storage, conn->GetNamespace());
+    auto s = redis.MDel(keys, &cnt);
+    if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
+
     *output = redis::Integer(cnt);
     return Status::OK();
   }
@@ -250,6 +276,7 @@ class CommandDel : public Commander {
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandTTL>("ttl", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandPTTL>("pttl", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandType>("type", 2, "read-only", 1, 1, 1),
+                        MakeCmdAttr<CommandMove>("move", 3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandObject>("object", 3, "read-only", 2, 2, 1),
                         MakeCmdAttr<CommandExists>("exists", -2, "read-only", 1, -1, 1),
                         MakeCmdAttr<CommandPersist>("persist", 2, "write", 1, 1, 1),
