@@ -19,8 +19,11 @@
  */
 
 #include "commander.h"
+#include "commands/blocking_commander.h"
+#include "commands/command_parser.h"
 #include "error_constants.h"
 #include "event_util.h"
+#include "server/redis_reply.h"
 #include "server/server.h"
 #include "types/redis_list.h"
 
@@ -153,9 +156,85 @@ class CommandRPop : public CommandPop {
   CommandRPop() : CommandPop(false) {}
 };
 
-class CommandBPop : public Commander,
-                    private EvbufCallbackBase<CommandBPop, false>,
-                    private EventCallbackBase<CommandBPop> {
+class CommandLMPop : public Commander {
+ public:
+  // format: LMPOP #numkeys key0 [key1 ...] <LEFT | RIGHT> [COUNT count]
+  Status Parse(const std::vector<std::string> &args) override {
+    CommandParser parser(args, 1);
+
+    auto num_keys = GET_OR_RET(parser.TakeInt<uint32_t>());
+    keys_.clear();
+    keys_.reserve(num_keys);
+    for (uint32_t i = 0; i < num_keys; ++i) {
+      keys_.emplace_back(GET_OR_RET(parser.TakeStr()));
+    }
+
+    auto left_or_right = util::ToLower(GET_OR_RET(parser.TakeStr()));
+    if (left_or_right == "left") {
+      left_ = true;
+    } else if (left_or_right == "right") {
+      left_ = false;
+    } else {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+
+    while (parser.Good()) {
+      if (parser.EatEqICase("count") && count_ == static_cast<uint32_t>(-1)) {
+        count_ = GET_OR_RET(parser.TakeInt<uint32_t>());
+      } else {
+        return parser.InvalidSyntax();
+      }
+    }
+    if (count_ == static_cast<uint32_t>(-1)) {
+      count_ = 1;
+    }
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    redis::List list_db(svr->storage, conn->GetNamespace());
+
+    std::vector<std::string> elems;
+    std::string chosen_key;
+    for (auto &key : keys_) {
+      auto s = list_db.PopMulti(key, left_, count_, &elems);
+      if (!s.ok() && !s.IsNotFound()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
+      if (!elems.empty()) {
+        chosen_key = key;
+        break;
+      }
+    }
+
+    if (elems.empty()) {
+      *output = redis::NilString();
+    } else {
+      std::string elems_bulk = redis::MultiBulkString(elems);
+      *output = redis::Array({redis::BulkString(chosen_key), std::move(elems_bulk)});
+    }
+
+    return Status::OK();
+  }
+
+  static const inline CommandKeyRangeGen keyRangeGen = [](const std::vector<std::string> &args) {
+    CommandKeyRange range;
+    range.first_key = 2;
+    range.key_step = 1;
+    // This parsing would always succeed as this cmd has been parsed before.
+    auto num_key = *ParseInt<int32_t>(args[1], 10);
+    range.last_key = range.first_key + num_key - 1;
+    return range;
+  };
+
+ private:
+  bool left_;
+  uint32_t count_ = -1;
+  std::vector<std::string> keys_;
+};
+
+class CommandBPop : public BlockingCommander {
  public:
   explicit CommandBPop(bool left) : left_(left) {}
 
@@ -182,34 +261,26 @@ class CommandBPop : public Commander,
 
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     svr_ = svr;
-    conn_ = conn;
+    InitConnection(conn);
 
-    auto bev = conn->GetBufferEvent();
     auto s = TryPopFromList();
     if (s.ok() || !s.IsNotFound()) {
       return Status::OK();  // error has already output in TryPopFromList
     }
 
-    if (conn->IsInExec()) {
-      *output = redis::MultiLen(-1);
-      return Status::OK();  // No blocking in multi-exec
-    }
+    return StartBlocking(timeout_, output);
+  }
 
+  void BlockKeys() override {
     for (const auto &key : keys_) {
       svr_->BlockOnKey(key, conn_);
     }
+  }
 
-    SetCB(bev);
-
-    if (timeout_) {
-      timer_.reset(NewTimer(bufferevent_get_base(bev)));
-      int64_t timeout_second = timeout_ / 1000 / 1000;
-      int64_t timeout_microsecond = timeout_ % (1000 * 1000);
-      timeval tm = {timeout_second, static_cast<int>(timeout_microsecond)};
-      evtimer_add(timer_.get(), &tm);
+  void UnblockKeys() override {
+    for (const auto &key : keys_) {
+      svr_->UnblockOnKey(key, conn_);
     }
-
-    return {Status::BlockingCmd};
   }
 
   rocksdb::Status TryPopFromList() {
@@ -239,62 +310,18 @@ class CommandBPop : public Commander,
     return s;
   }
 
-  void OnWrite(bufferevent *bev) {
+  bool OnBlockingWrite() override {
     auto s = TryPopFromList();
-    if (s.IsNotFound()) {
-      // The connection may be waked up but can't pop from list. For example,
-      // connection A is blocking on list and connection B push a new element
-      // then wake up the connection A, but this element may be token by other connection C.
-      // So we need to wait for the wake event again by disabling the WRITE event.
-      bufferevent_disable(bev, EV_WRITE);
-      return;
-    }
-
-    if (timer_) {
-      timer_.reset();
-    }
-
-    unBlockingAll();
-    conn_->SetCB(bev);
-    bufferevent_enable(bev, EV_READ);
-    // We need to manually trigger the read event since we will stop processing commands
-    // in connection after the blocking command, so there may have some commands to be processed.
-    // Related issue: https://github.com/apache/kvrocks/issues/831
-    bufferevent_trigger(bev, EV_READ, BEV_TRIG_IGNORE_WATERMARKS);
+    return !s.IsNotFound();
   }
 
-  void OnEvent(bufferevent *bev, int16_t events) {
-    if (events & (BEV_EVENT_EOF | BEV_EVENT_ERROR)) {
-      if (timer_ != nullptr) {
-        timer_.reset();
-      }
-      unBlockingAll();
-    }
-    conn_->OnEvent(bev, events);
-  }
-
-  void TimerCB(int, int16_t events) {
-    conn_->Reply(redis::NilString());
-    timer_.reset();
-    unBlockingAll();
-    auto bev = conn_->GetBufferEvent();
-    conn_->SetCB(bev);
-    bufferevent_enable(bev, EV_READ);
-  }
+  std::string NoopReply() override { return redis::NilString(); }
 
  private:
   bool left_ = false;
   int64_t timeout_ = 0;  // microseconds
   std::vector<std::string> keys_;
   Server *svr_ = nullptr;
-  Connection *conn_ = nullptr;
-  UniqueEvent timer_;
-
-  void unBlockingAll() {
-    for (const auto &key : keys_) {
-      svr_->UnblockOnKey(key, conn_);
-    }
-  }
 };
 
 class CommandBLPop : public CommandBPop {
@@ -508,7 +535,7 @@ class CommandRPopLPUSH : public Commander {
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     redis::List list_db(svr->storage, conn->GetNamespace());
     std::string elem;
-    auto s = list_db.RPopLPush(args_[1], args_[2], &elem);
+    auto s = list_db.LMove(args_[1], args_[2], /*src_left=*/false, /*dst_left=*/true, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -553,19 +580,167 @@ class CommandLMove : public Commander {
   bool dst_left_;
 };
 
+class CommandBLMove : public BlockingCommander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    auto arg_val = util::ToLower(args_[3]);
+    if (arg_val != "left" && arg_val != "right") {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+
+    src_left_ = arg_val == "left";
+    arg_val = util::ToLower(args_[4]);
+    if (arg_val != "left" && arg_val != "right") {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+    dst_left_ = arg_val == "left";
+
+    auto parse_result = ParseFloat(args[args.size() - 1]);
+    if (!parse_result) {
+      return {Status::RedisParseErr, errTimeoutIsNotFloat};
+    }
+    if (*parse_result < 0) {
+      return {Status::RedisParseErr, errTimeoutIsNegative};
+    }
+    timeout_ = static_cast<int64_t>(*parse_result * 1000 * 1000);
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    svr_ = svr;
+    InitConnection(conn);
+
+    redis::List list_db(svr->storage, conn->GetNamespace());
+    std::string elem;
+    auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
+    if (!s.ok() && !s.IsNotFound()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+    if (!elem.empty()) {
+      *output = redis::BulkString(elem);
+      return Status::OK();
+    }
+
+    return StartBlocking(timeout_, output);
+  }
+
+  void BlockKeys() override { svr_->BlockOnKey(args_[1], conn_); }
+
+  void UnblockKeys() override { svr_->UnblockOnKey(args_[1], conn_); }
+
+  bool OnBlockingWrite() override {
+    redis::List list_db(svr_->storage, conn_->GetNamespace());
+    std::string elem;
+    auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
+    if (!s.ok() && !s.IsNotFound()) {
+      conn_->Reply(redis::Error("ERR " + s.ToString()));
+      return true;
+    }
+
+    bool empty = elem.empty();
+    if (!empty) {
+      conn_->Reply(redis::BulkString(elem));
+    }
+
+    return !empty;
+  }
+
+  std::string NoopReply() override { return redis::MultiLen(-1); }
+
+ private:
+  bool src_left_;
+  bool dst_left_;
+  int64_t timeout_ = 0;  // microseconds
+  Server *svr_ = nullptr;
+};
+
+class CommandLPos : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    if (args.size() > 9) {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+
+    CommandParser parser(args, 3);
+    while (parser.Good()) {
+      if (parser.EatEqICase("rank")) {
+        spec_.rank = GET_OR_RET(parser.TakeInt());
+        if (spec_.rank == 0) {
+          return {Status::RedisParseErr,
+                  "RANK can't be zero: use 1 to start from "
+                  "the first match, 2 from the second ... "
+                  "or use negative to start from the end of the list"};
+        } else if (spec_.rank == LLONG_MIN) {
+          // Negating LLONG_MIN will cause an overflow, and is effectively be the same as passing -1.
+          return {Status::RedisParseErr, "rank would overflow"};
+        }
+      } else if (parser.EatEqICase("count")) {
+        spec_.count = GET_OR_RET(parser.TakeInt());
+        if (spec_.count < 0) {
+          return {Status::RedisParseErr, "COUNT can't be negative"};
+        }
+      } else if (parser.EatEqICase("maxlen")) {
+        spec_.max_len = GET_OR_RET(parser.TakeInt());
+        if (spec_.max_len < 0) {
+          return {Status::RedisParseErr, "MAXLEN can't be negative"};
+        }
+      } else {
+        return {Status::RedisParseErr, errInvalidSyntax};
+      }
+    }
+    return Status::OK();
+  }
+
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    redis::List list_db(svr->storage, conn->GetNamespace());
+    std::vector<int64_t> indexes;
+    auto s = list_db.Pos(args_[1], args_[2], spec_, &indexes);
+    if (!s.ok() && !s.IsNotFound()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+
+    // We return nil or a single value if `COUNT` option is not given.
+    if (!spec_.count.has_value()) {
+      if (s.IsNotFound() || indexes.empty()) {
+        *output = redis::NilString();
+      } else {
+        assert(indexes.size() == 1);
+        *output = redis::Integer(indexes[0]);
+      }
+    }
+    // Otherwise we return an array.
+    else {
+      std::vector<std::string> values;
+      values.reserve(indexes.size());
+      for (const auto &index : indexes) {
+        values.emplace_back(std::to_string(index));
+      }
+      *output = redis::MultiBulkString(values, false);
+    }
+    return Status::OK();
+  }
+
+ private:
+  PosSpec spec_;
+};
+
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandBLPop>("blpop", -3, "write no-script", 1, -2, 1),
                         MakeCmdAttr<CommandBRPop>("brpop", -3, "write no-script", 1, -2, 1),
                         MakeCmdAttr<CommandLIndex>("lindex", 3, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLInsert>("linsert", 5, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLLen>("llen", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLMove>("lmove", 5, "write", 1, 2, 1),
+                        MakeCmdAttr<CommandBLMove>("blmove", 6, "write", 1, 2, 1),
                         MakeCmdAttr<CommandLPop>("lpop", -2, "write", 1, 1, 1),  //
+                        MakeCmdAttr<CommandLPos>("lpos", -3, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLPush>("lpush", -3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLPushX>("lpushx", -3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLRange>("lrange", 4, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLRem>("lrem", 4, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLSet>("lset", 4, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLTrim>("ltrim", 4, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandLMPop>("lmpop", -4, "write", CommandLMPop::keyRangeGen),
                         MakeCmdAttr<CommandRPop>("rpop", -2, "write", 1, 1, 1),
                         MakeCmdAttr<CommandRPopLPUSH>("rpoplpush", 3, "write", 1, 2, 1),
                         MakeCmdAttr<CommandRPush>("rpush", -3, "write", 1, 1, 1),
