@@ -18,6 +18,7 @@
  *
  */
 
+#include "command_parser.h"
 #include "commander.h"
 #include "commands/scan_base.h"
 #include "common/io_util.h"
@@ -26,6 +27,8 @@
 #include "server/redis_connection.h"
 #include "server/server.h"
 #include "stats/disk_stats.h"
+#include "storage/rdb.h"
+#include "string_util.h"
 #include "time_util.h"
 
 namespace redis {
@@ -69,11 +72,9 @@ class CommandAuth : public Commander {
         *output = redis::SimpleString("OK");
         break;
       case AuthResult::INVALID_PASSWORD:
-        *output = redis::Error("ERR invalid password");
-        break;
+        return {Status::RedisExecErr, "invalid password"};
       case AuthResult::NO_REQUIRE_PASS:
-        *output = redis::Error("ERR Client sent AUTH, but no password is set");
-        break;
+        return {Status::RedisExecErr, "Client sent AUTH, but no password is set"};
     }
     return Status::OK();
   }
@@ -83,8 +84,7 @@ class CommandNamespace : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     Config *config = svr->GetConfig();
@@ -111,20 +111,20 @@ class CommandNamespace : public Commander {
       }
     } else if (args_.size() == 4 && sub_command == "set") {
       Status s = config->SetNamespace(args_[2], args_[3]);
-      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error(s.Msg());
+      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error("ERR " + s.Msg());
       LOG(WARNING) << "Updated namespace: " << args_[2] << " with token: " << args_[3] << ", addr: " << conn->GetAddr()
                    << ", result: " << s.Msg();
     } else if (args_.size() == 4 && sub_command == "add") {
       Status s = config->AddNamespace(args_[2], args_[3]);
-      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error(s.Msg());
+      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error("ERR " + s.Msg());
       LOG(WARNING) << "New namespace: " << args_[2] << " with token: " << args_[3] << ", addr: " << conn->GetAddr()
                    << ", result: " << s.Msg();
     } else if (args_.size() == 3 && sub_command == "del") {
       Status s = config->DelNamespace(args_[2]);
-      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error(s.Msg());
+      *output = s.IsOK() ? redis::SimpleString("OK") : redis::Error("ERR " + s.Msg());
       LOG(WARNING) << "Deleted namespace: " << args_[2] << ", addr: " << conn->GetAddr() << ", result: " << s.Msg();
     } else {
-      *output = redis::Error("NAMESPACE subcommand must be one of GET, SET, DEL, ADD");
+      return {Status::RedisExecErr, "NAMESPACE subcommand must be one of GET, SET, DEL, ADD"};
     }
     return Status::OK();
   }
@@ -136,15 +136,19 @@ class CommandKeys : public Commander {
     std::string prefix = args_[1];
     std::vector<std::string> keys;
     redis::Database redis(svr->storage, conn->GetNamespace());
+
+    rocksdb::Status s;
     if (prefix == "*") {
-      redis.Keys(std::string(), &keys);
+      s = redis.Keys(std::string(), &keys);
     } else {
       if (prefix[prefix.size() - 1] != '*') {
-        *output = redis::Error("ERR only keys prefix match was supported");
-        return Status::OK();
+        return {Status::RedisExecErr, "only keys prefix match was supported"};
       }
 
-      redis.Keys(prefix.substr(0, prefix.size() - 1), &keys);
+      s = redis.Keys(prefix.substr(0, prefix.size() - 1), &keys);
+    }
+    if (!s.ok()) {
+      return {Status::RedisExecErr, s.ToString()};
     }
     *output = redis::MultiBulkString(keys);
     return Status::OK();
@@ -176,8 +180,7 @@ class CommandFlushAll : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     if (svr->GetConfig()->cluster_enabled) {
@@ -225,16 +228,14 @@ class CommandConfig : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     Config *config = svr->GetConfig();
     std::string sub_command = util::ToLower(args_[1]);
     if ((sub_command == "rewrite" && args_.size() != 2) || (sub_command == "get" && args_.size() != 3) ||
         (sub_command == "set" && args_.size() != 4)) {
-      *output = redis::Error(errWrongNumOfArguments);
-      return Status::OK();
+      return {Status::RedisExecErr, errWrongNumOfArguments};
     }
 
     if (args_.size() == 2 && sub_command == "rewrite") {
@@ -250,12 +251,12 @@ class CommandConfig : public Commander {
     } else if (args_.size() == 4 && sub_command == "set") {
       Status s = config->Set(svr, args_[2], args_[3]);
       if (!s.IsOK()) {
-        *output = redis::Error("CONFIG SET '" + args_[2] + "' error: " + s.Msg());
+        return {Status::RedisExecErr, "CONFIG SET '" + args_[2] + "' error: " + s.Msg()};
       } else {
         *output = redis::SimpleString("OK");
       }
     } else {
-      *output = redis::Error("CONFIG subcommand must be one of GET, SET, REWRITE");
+      return {Status::RedisExecErr, "CONFIG subcommand must be one of GET, SET, REWRITE"};
     }
     return Status::OK();
   }
@@ -267,6 +268,8 @@ class CommandInfo : public Commander {
     std::string section = "all";
     if (args_.size() == 2) {
       section = util::ToLower(args_[1]);
+    } else if (args_.size() > 2) {
+      return {Status::RedisParseErr, errInvalidSyntax};
     }
     std::string info;
     svr->GetInfo(conn->GetNamespace(), section, &info);
@@ -328,10 +331,10 @@ class CommandDBSize : public Commander {
       if (s.IsOK()) {
         *output = redis::SimpleString("OK");
       } else {
-        *output = redis::Error(s.Msg());
+        return {Status::RedisExecErr, s.Msg()};
       }
     } else {
-      *output = redis::Error("DBSIZE subcommand only supports scan");
+      return {Status::RedisExecErr, "DBSIZE subcommand only supports scan"};
     }
     return Status::OK();
   }
@@ -518,7 +521,7 @@ class CommandClient : public Commander {
         *output = redis::Integer(killed);
       } else {
         if (killed == 0)
-          *output = redis::Error("No such client");
+          return {Status::RedisExecErr, "No such client"};
         else
           *output = redis::SimpleString("OK");
       }
@@ -551,8 +554,7 @@ class CommandShutdown : public Commander {
  public:
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     if (!srv->IsStopped()) {
@@ -610,8 +612,7 @@ class CommandCommand : public Commander {
       std::string sub_command = util::ToLower(args_[1]);
       if ((sub_command == "count" && args_.size() != 2) || (sub_command == "getkeys" && args_.size() < 3) ||
           (sub_command == "info" && args_.size() < 3)) {
-        *output = redis::Error(errWrongNumOfArguments);
-        return Status::OK();
+        return {Status::RedisExecErr, errWrongNumOfArguments};
       }
 
       if (sub_command == "count") {
@@ -619,13 +620,18 @@ class CommandCommand : public Commander {
       } else if (sub_command == "info") {
         GetCommandsInfo(output, std::vector<std::string>(args_.begin() + 2, args_.end()));
       } else if (sub_command == "getkeys") {
+        auto cmd_iter = command_details::original_commands.find(util::ToLower(args_[2]));
+        if (cmd_iter == command_details::original_commands.end()) {
+          return {Status::RedisUnknownCmd, "Invalid command specified"};
+        }
+
         std::vector<int> keys_indexes;
-        auto s = GetKeysFromCommand(args_[2], static_cast<int>(args_.size()) - 2, &keys_indexes);
+        auto s = GetKeysFromCommand(cmd_iter->second, std::vector<std::string>(args_.begin() + 2, args_.end()),
+                                    &keys_indexes);
         if (!s.IsOK()) return s;
 
         if (keys_indexes.size() == 0) {
-          *output = redis::Error("Invalid arguments specified for command");
-          return Status::OK();
+          return {Status::RedisExecErr, "Invalid arguments specified for command"};
         }
 
         std::vector<std::string> keys;
@@ -635,7 +641,7 @@ class CommandCommand : public Commander {
         }
         *output = redis::MultiBulkString(keys);
       } else {
-        *output = redis::Error("Command subcommand must be one of COUNT, GETKEYS, INFO");
+        return {Status::RedisExecErr, "Command subcommand must be one of COUNT, GETKEYS, INFO"};
       }
     }
     return Status::OK();
@@ -718,8 +724,7 @@ class CommandHello final : public Commander {
         conn->SetName(name);
         next_arg += 1;
       } else {
-        *output = redis::Error("Syntax error in HELLO option " + opt);
-        return Status::OK();
+        return {Status::RedisExecErr, "Syntax error in HELLO option " + opt};
       }
     }
 
@@ -802,7 +807,10 @@ class CommandRandomKey : public Commander {
     std::string key;
     auto cursor = svr->GetLastRandomKeyCursor();
     redis::Database redis(svr->storage, conn->GetNamespace());
-    redis.RandomKey(cursor, &key);
+    auto s = redis.RandomKey(cursor, &key);
+    if (!s.ok()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
     svr->SetLastRandomKeyCursor(key);
     *output = redis::BulkString(key);
     return Status::OK();
@@ -816,8 +824,7 @@ class CommandCompact : public Commander {
     auto ns = conn->GetNamespace();
 
     if (ns != kDefaultNamespace) {
-      std::string prefix;
-      ComposeNamespaceKey(ns, "", &prefix, false);
+      std::string prefix = ComposeNamespaceKey(ns, "", false);
 
       redis::Database redis_db(svr->storage, conn->GetNamespace());
       auto s = redis_db.FindKeyRangeWithPrefix(prefix, std::string(), &begin_key, &end_key);
@@ -844,8 +851,7 @@ class CommandBGSave : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     Status s = svr->AsyncBgSaveDB();
@@ -861,8 +867,7 @@ class CommandFlushBackup : public Commander {
  public:
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     Status s = svr->AsyncPurgeOldBackups(0, 0);
@@ -921,8 +926,7 @@ class CommandSlaveOf : public Commander {
     }
 
     if (!conn->IsAdmin()) {
-      *output = redis::Error(errAdministorPermissionRequired);
-      return Status::OK();
+      return {Status::RedisExecErr, errAdminPermissionRequired};
     }
 
     if (host_.empty()) {
@@ -976,13 +980,104 @@ class CommandStats : public Commander {
   }
 };
 
+static uint64_t GenerateConfigFlag(const std::vector<std::string> &args) {
+  if (args.size() >= 2 && util::EqualICase(args[1], "set")) {
+    return kCmdExclusive;
+  }
+
+  return 0;
+}
+
+class CommandLastSave : public Commander {
+ public:
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    if (!conn->IsAdmin()) {
+      return {Status::RedisExecErr, errAdminPermissionRequired};
+    }
+
+    int64_t unix_sec = svr->GetLastBgsaveTime();
+    *output = redis::Integer(unix_sec);
+    return Status::OK();
+  }
+};
+
+class CommandRestore : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    CommandParser parser(args, 4);
+    ttl_ms_ = GET_OR_RET(ParseInt<int64_t>(args[2], {0, INT64_MAX}, 10));
+    while (parser.Good()) {
+      if (parser.EatEqICase("replace")) {
+        replace_ = true;
+      } else if (parser.EatEqICase("absttl")) {
+        absttl_ = true;
+      } else if (parser.EatEqICase("idletime")) {
+        // idle time is not supported in Kvrocks, so just skip it
+        auto idle_time = GET_OR_RET(parser.TakeInt());
+        if (idle_time < 0) {
+          return {Status::RedisParseErr, "IDLETIME can't be negative"};
+        }
+      } else if (parser.EatEqICase("freq")) {
+        // freq is not supported in Kvrocks, so just skip it
+        auto freq = GET_OR_RET(parser.TakeInt());
+        if (freq < 0 || freq > 255) {
+          return {Status::RedisParseErr, "FREQ must be >= 0 and <= 255"};
+        }
+      } else {
+        return {Status::RedisParseErr, errInvalidSyntax};
+      }
+    }
+    return Status::OK();
+  }
+
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    rocksdb::Status db_status;
+    redis::Database redis(svr->storage, conn->GetNamespace());
+    if (!replace_) {
+      int count = 0;
+      db_status = redis.Exists({args_[1]}, &count);
+      if (!db_status.ok()) {
+        return {Status::RedisExecErr, db_status.ToString()};
+      }
+      if (count > 0) {
+        return {Status::RedisExecErr, "target key name already exists."};
+      }
+    } else {
+      db_status = redis.Del(args_[1]);
+      if (!db_status.ok() && !db_status.IsNotFound()) {
+        return {Status::RedisExecErr, db_status.ToString()};
+      }
+    }
+    if (ttl_ms_ && absttl_) {
+      auto now = util::GetTimeStampMS();
+      if (ttl_ms_ <= now) {
+        // return ok if the ttl is already expired
+        *output = redis::SimpleString("OK");
+        return Status::OK();
+      }
+      ttl_ms_ -= now;
+    }
+
+    RDB rdb(svr->storage, conn->GetNamespace(), args_[3]);
+    auto s = rdb.Restore(args_[1], ttl_ms_);
+    if (!s.IsOK()) return {Status::RedisExecErr, s.Msg()};
+    *output = redis::SimpleString("OK");
+    return Status::OK();
+  }
+
+ private:
+  bool replace_ = false;
+  bool absttl_ = false;
+  uint64_t ttl_ms_ = 0;
+};
+
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandAuth>("auth", 2, "read-only ok-loading", 0, 0, 0),
                         MakeCmdAttr<CommandPing>("ping", -1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandSelect>("select", 2, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandInfo>("info", -1, "read-only ok-loading", 0, 0, 0),
                         MakeCmdAttr<CommandRole>("role", 1, "read-only ok-loading", 0, 0, 0),
-                        MakeCmdAttr<CommandConfig>("config", -2, "read-only", 0, 0, 0),
-                        MakeCmdAttr<CommandNamespace>("namespace", -3, "read-only", 0, 0, 0),
+                        MakeCmdAttr<CommandConfig>("config", -2, "read-only", 0, 0, 0, GenerateConfigFlag),
+                        MakeCmdAttr<CommandNamespace>("namespace", -3, "read-only exclusive", 0, 0, 0),
                         MakeCmdAttr<CommandKeys>("keys", 2, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandFlushDB>("flushdb", 1, "write", 0, 0, 0),
                         MakeCmdAttr<CommandFlushAll>("flushall", 1, "write", 0, 0, 0),
@@ -994,7 +1089,7 @@ REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandAuth>("auth", 2, "read-only ok-loadin
                         MakeCmdAttr<CommandShutdown>("shutdown", 1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandQuit>("quit", 1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandScan>("scan", -2, "read-only", 0, 0, 0),
-                        MakeCmdAttr<CommandRandomKey>("randomkey", 1, "read-only no-script", 0, 0, 0),
+                        MakeCmdAttr<CommandRandomKey>("randomkey", 1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandDebug>("debug", -2, "read-only exclusive", 0, 0, 0),
                         MakeCmdAttr<CommandCommand>("command", -1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandEcho>("echo", 2, "read-only", 0, 0, 0),
@@ -1002,9 +1097,11 @@ REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandAuth>("auth", 2, "read-only ok-loadin
                         MakeCmdAttr<CommandDisk>("disk", 3, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandMemory>("memory", 3, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandHello>("hello", -1, "read-only ok-loading", 0, 0, 0),
+                        MakeCmdAttr<CommandRestore>("restore", -4, "write", 1, 1, 1),
 
                         MakeCmdAttr<CommandCompact>("compact", 1, "read-only no-script", 0, 0, 0),
                         MakeCmdAttr<CommandBGSave>("bgsave", 1, "read-only no-script", 0, 0, 0),
+                        MakeCmdAttr<CommandLastSave>("lastsave", 1, "read-only", 0, 0, 0),
                         MakeCmdAttr<CommandFlushBackup>("flushbackup", 1, "read-only no-script", 0, 0, 0),
                         MakeCmdAttr<CommandSlaveOf>("slaveof", 3, "read-only exclusive no-script", 0, 0, 0),
                         MakeCmdAttr<CommandStats>("stats", 1, "read-only", 0, 0, 0), )
