@@ -25,6 +25,7 @@
 #include "commands/command_parser.h"
 #include "error_constants.h"
 #include "server/server.h"
+#include "storage/redis_db.h"
 #include "time_util.h"
 #include "ttl_util.h"
 #include "types/redis_bitmap.h"
@@ -86,6 +87,13 @@ class CommandGetEx : public Commander {
       uint32_t max_btos_size = static_cast<uint32_t>(config->max_bitmap_to_string_mb) * MiB;
       redis::Bitmap bitmap_db(svr->storage, conn->GetNamespace());
       s = bitmap_db.GetString(args_[1], max_btos_size, &value);
+      if (s.ok()) {
+        if (ttl_ > 0) {
+          s = bitmap_db.Expire(args_[1], ttl_ + util::GetTimeStampMS());
+        } else if (persist_) {
+          s = bitmap_db.Expire(args_[1], 0);
+        }
+      }
     }
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -288,14 +296,17 @@ class CommandSet : public Commander {
   Status Execute(Server *svr, Connection *conn, std::string *output) override {
     bool ret = false;
     redis::String string_db(svr->storage, conn->GetNamespace());
-    rocksdb::Status s;
 
     if (ttl_ < 0) {
-      string_db.Del(args_[1]);
+      auto s = string_db.Del(args_[1]);
+      if (!s.ok()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
       *output = redis::SimpleString("OK");
       return Status::OK();
     }
 
+    rocksdb::Status s;
     if (set_flag_ == NX) {
       s = string_db.SetNX(args_[1], args_[2], ttl_, &ret);
     } else if (set_flag_ == XX) {
@@ -559,10 +570,8 @@ class CommandCAS : public Commander {
     CommandParser parser(args, 4);
     std::string_view flag;
     while (parser.Good()) {
-      if (parser.EatEqICaseFlag("EX", flag)) {
-        ttl_ = GET_OR_RET(parser.TakeInt<int64_t>(TTL_RANGE<int64_t>)) * 1000;
-      } else if (parser.EatEqICaseFlag("PX", flag)) {
-        ttl_ = GET_OR_RET(parser.TakeInt<int64_t>(TTL_RANGE<int64_t>));
+      if (auto v = GET_OR_RET(ParseTTL(parser, flag))) {
+        ttl_ = *v;
       } else {
         return parser.InvalidSyntax();
       }
@@ -613,7 +622,7 @@ REDIS_REGISTER_COMMANDS(
     MakeCmdAttr<CommandAppend>("append", 3, "write", 1, 1, 1), MakeCmdAttr<CommandSet>("set", -3, "write", 1, 1, 1),
     MakeCmdAttr<CommandSetEX>("setex", 4, "write", 1, 1, 1), MakeCmdAttr<CommandPSetEX>("psetex", 4, "write", 1, 1, 1),
     MakeCmdAttr<CommandSetNX>("setnx", 3, "write", 1, 1, 1),
-    MakeCmdAttr<CommandMSetNX>("msetnx", -3, "write exclusive", 1, -1, 2),
+    MakeCmdAttr<CommandMSetNX>("msetnx", -3, "write", 1, -1, 2),
     MakeCmdAttr<CommandMSet>("mset", -3, "write", 1, -1, 2), MakeCmdAttr<CommandIncrBy>("incrby", 3, "write", 1, 1, 1),
     MakeCmdAttr<CommandIncrByFloat>("incrbyfloat", 3, "write", 1, 1, 1),
     MakeCmdAttr<CommandIncr>("incr", 2, "write", 1, 1, 1), MakeCmdAttr<CommandDecrBy>("decrby", 3, "write", 1, 1, 1),
