@@ -50,7 +50,7 @@ namespace redis {
 class Connection;
 struct CommandAttributes;
 
-enum CommandFlags {
+enum CommandFlags : uint64_t {
   kCmdWrite = 1ULL << 0,        // "write" flag
   kCmdReadOnly = 1ULL << 1,     // "read-only" flag
   kCmdReplication = 1ULL << 2,  // "replication" flag
@@ -68,7 +68,7 @@ enum CommandFlags {
 class Commander {
  public:
   void SetAttributes(const CommandAttributes *attributes) { attributes_ = attributes; }
-  const CommandAttributes *GetAttributes() { return attributes_; }
+  const CommandAttributes *GetAttributes() const { return attributes_; }
   void SetArgs(const std::vector<std::string> &args) { args_ = args; }
   virtual Status Parse() { return Parse(args_); }
   virtual Status Parse(const std::vector<std::string> &args) { return Status::OK(); }
@@ -113,7 +113,12 @@ struct CommandKeyRange {
 
 using CommandKeyRangeGen = std::function<CommandKeyRange(const std::vector<std::string> &)>;
 
+using CommandKeyRangeVecGen = std::function<std::vector<CommandKeyRange>(const std::vector<std::string> &)>;
+
+using AdditionalFlagGen = std::function<uint64_t(const std::vector<std::string> &)>;
+
 struct CommandAttributes {
+  // command name
   std::string name;
 
   // number of command arguments
@@ -121,21 +126,32 @@ struct CommandAttributes {
   // negative number -n means number of arguments is equal to or large than n
   int arity;
 
+  // space-separated flag strings to initialize flags
   std::string description;
+
+  // bitmap of enum CommandFlags
   uint64_t flags;
 
+  // additional flags regarding to dynamic command arguments
+  AdditionalFlagGen flag_gen;
+
+  // static determined key range
   CommandKeyRange key_range;
 
   // if key_range.first_key == -1, key_range_gen is used instead
   CommandKeyRangeGen key_range_gen;
 
+  // if key_range.first_key == -2, key_range_vec_gen is used instead
+  CommandKeyRangeVecGen key_range_vec_gen;
+
+  // commander object generator
   CommanderFactory factory;
 
-  bool IsWrite() const { return (flags & kCmdWrite) != 0; }
-  bool IsOkLoading() const { return (flags & kCmdLoading) != 0; }
-  bool IsExclusive() const { return (flags & kCmdExclusive) != 0; }
-  bool IsMulti() const { return (flags & kCmdMulti) != 0; }
-  bool IsNoMulti() const { return (flags & kCmdNoMulti) != 0; }
+  auto GenerateFlags(const std::vector<std::string> &args) const {
+    uint64_t res = flags;
+    if (flag_gen) res |= flag_gen(args);
+    return res;
+  }
 };
 
 using CommandMap = std::map<std::string, const CommandAttributes *>;
@@ -179,13 +195,15 @@ inline uint64_t ParseCommandFlags(const std::string &description, const std::str
 
 template <typename T>
 auto MakeCmdAttr(const std::string &name, int arity, const std::string &description, int first_key, int last_key,
-                 int key_step) {
+                 int key_step, const AdditionalFlagGen &flag_gen = {}) {
   CommandAttributes attr{name,
                          arity,
                          description,
                          ParseCommandFlags(description, name),
+                         flag_gen,
                          {first_key, last_key, key_step},
                          {},
+                         {},
                          []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
 
   if ((first_key > 0 && key_step <= 0) || (first_key > 0 && last_key >= 0 && last_key < first_key)) {
@@ -197,13 +215,32 @@ auto MakeCmdAttr(const std::string &name, int arity, const std::string &descript
 }
 
 template <typename T>
-auto MakeCmdAttr(const std::string &name, int arity, const std::string &description, const CommandKeyRangeGen &gen) {
+auto MakeCmdAttr(const std::string &name, int arity, const std::string &description, const CommandKeyRangeGen &gen,
+                 const AdditionalFlagGen &flag_gen = {}) {
   CommandAttributes attr{name,
                          arity,
                          description,
                          ParseCommandFlags(description, name),
+                         flag_gen,
                          {-1, 0, 0},
                          gen,
+                         {},
+                         []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
+
+  return attr;
+}
+
+template <typename T>
+auto MakeCmdAttr(const std::string &name, int arity, const std::string &description,
+                 const CommandKeyRangeVecGen &vec_gen, const AdditionalFlagGen &flag_gen = {}) {
+  CommandAttributes attr{name,
+                         arity,
+                         description,
+                         ParseCommandFlags(description, name),
+                         flag_gen,
+                         {-2, 0, 0},
+                         {},
+                         vec_gen,
                          []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
 
   return attr;
@@ -231,14 +268,15 @@ inline CommandMap commands;
 #define REDIS_REGISTER_COMMANDS(...) \
   static RegisterToCommandTable KVROCKS_CONCAT2(register_to_command_table_, __LINE__){__VA_ARGS__};
 
-int GetCommandNum();
+size_t GetCommandNum();
 CommandMap *GetCommands();
 void ResetCommands();
 const CommandMap *GetOriginalCommands();
 void GetAllCommandsInfo(std::string *info);
 void GetCommandsInfo(std::string *info, const std::vector<std::string> &cmd_names);
 std::string GetCommandInfo(const CommandAttributes *command_attributes);
-Status GetKeysFromCommand(const std::string &name, int argc, std::vector<int> *keys_indexes);
+Status GetKeysFromCommand(const CommandAttributes *attributes, const std::vector<std::string> &cmd_tokens,
+                          std::vector<int> *keys_indexes);
 bool IsCommandExists(const std::string &name);
 
 }  // namespace redis
