@@ -22,6 +22,7 @@
 
 #include <fmt/format.h>
 
+#include <regex>
 #include <string>
 
 #include "parse_util.h"
@@ -214,6 +215,19 @@ int StringMatchLen(const char *pattern, size_t pattern_len, const char *string,
   return 0;
 }
 
+std::vector<std::string> RegexMatch(const std::string &str, const std::string &regex) {
+  std::regex base_regex(regex);
+  std::smatch pieces_match;
+  std::vector<std::string> out;
+
+  if (std::regex_match(str, pieces_match, base_regex)) {
+    for (const auto &piece : pieces_match) {
+      out.emplace_back(piece.str());
+    }
+  }
+  return out;
+}
+
 std::string StringToHex(const std::string &input) {
   static const char hex_digits[] = "0123456789ABCDEF";
   std::string output;
