@@ -40,7 +40,6 @@
 #include "commands/commander.h"
 #include "config.h"
 #include "fmt/format.h"
-#include "lua.h"
 #include "redis_connection.h"
 #include "storage/compaction_checker.h"
 #include "storage/redis_db.h"
@@ -67,7 +66,7 @@ Server::Server(engine::Storage *storage, Config *config)
 
 #ifdef ENABLE_OPENSSL
   // init ssl context
-  if (config->tls_port) {
+  if (config->tls_port || config->tls_replication) {
     ssl_ctx = CreateSSLContext(config);
     if (!ssl_ctx) {
       exit(1);
@@ -117,7 +116,6 @@ Server::~Server() {
   }
 
   lua::DestroyState(lua_);
-  libevent_global_shutdown();
 }
 
 // Kvrocks threads list:
@@ -144,15 +142,17 @@ Status Server::Start() {
     }
   }
 
-  if (config_->cluster_enabled && config_->persist_cluster_nodes_enabled) {
-    auto s = cluster->LoadClusterNodes(config_->NodesFilePath());
-    if (!s.IsOK()) {
-      return s.Prefixed("failed to load cluster nodes info");
+  if (config_->cluster_enabled) {
+    if (config_->persist_cluster_nodes_enabled) {
+      auto s = cluster->LoadClusterNodes(config_->NodesFilePath());
+      if (!s.IsOK()) {
+        return s.Prefixed("failed to load cluster nodes info");
+      }
     }
     // Create objects used for slot migration
     slot_migrator =
         std::make_unique<SlotMigrator>(this, config_->migrate_speed, config_->pipeline_size, config_->sequence_gap);
-    s = slot_migrator->CreateMigrationThread();
+    auto s = slot_migrator->CreateMigrationThread();
     if (!s.IsOK()) {
       return s.Prefixed("failed to create migration thread");
     }
@@ -339,19 +339,17 @@ void Server::FeedMonitorConns(redis::Connection *conn, const std::vector<std::st
   if (monitor_clients_ <= 0) return;
 
   auto now = util::GetTimeStampUS();
-  std::string output = "+";
-  output += std::to_string(now / 1000000) + "." + std::to_string(now % 1000000);
-  output += " [" + conn->GetNamespace() + " " + conn->GetAddr() + "]";
+  std::string output =
+      fmt::format("{}.{} [{} {}]", now / 1000000, now % 1000000, conn->GetNamespace(), conn->GetAddr());
   for (const auto &token : tokens) {
     output += " \"";
     output += util::EscapeString(token);
     output += "\"";
   }
-  output += CRLF;
 
   for (const auto &worker_thread : worker_threads_) {
     auto worker = worker_thread->GetWorker();
-    worker->FeedMonitorConns(conn, output);
+    worker->FeedMonitorConns(conn, redis::SimpleString(output));
   }
 }
 
@@ -621,11 +619,7 @@ void Server::OnEntryAddedToStream(const std::string &ns, const std::string &key,
   }
 }
 
-void Server::updateCachedTime() {
-  time_t ret = util::GetTimeStamp();
-  if (ret == -1) return;
-  unix_time.store(static_cast<int>(ret));
-}
+void Server::updateCachedTime() { unix_time.store(util::GetTimeStamp()); }
 
 int Server::IncrClientNum() {
   total_clients_.fetch_add(1, std::memory_order::memory_order_relaxed);
@@ -914,7 +908,7 @@ void Server::GetReplicationInfo(std::string *info) {
     if (slave->IsStopped()) continue;
 
     string_stream << "slave" << std::to_string(idx) << ":";
-    string_stream << "ip=" << slave->GetConn()->GetAnnounceIP() << ",port=" << slave->GetConn()->GetListeningPort()
+    string_stream << "ip=" << slave->GetConn()->GetAnnounceIP() << ",port=" << slave->GetConn()->GetAnnouncePort()
                   << ",offset=" << slave->GetCurrentReplSeq() << ",lag=" << latest_seq - slave->GetCurrentReplSeq()
                   << "\r\n";
     ++idx;
@@ -983,13 +977,18 @@ void Server::SetLastRandomKeyCursor(const std::string &cursor) {
   last_random_key_cursor_ = cursor;
 }
 
-int Server::GetCachedUnixTime() {
+int64_t Server::GetCachedUnixTime() {
   if (unix_time.load() == 0) {
-    unix_time.store(static_cast<int>(util::GetTimeStamp()));
+    updateCachedTime();
   }
   return unix_time.load();
 }
 
+int64_t Server::GetLastBgsaveTime() {
+  std::lock_guard<std::mutex> lg(db_job_mu_);
+  return last_bgsave_time_ == -1 ? start_time_ : last_bgsave_time_;
+}
+
 void Server::GetStatsInfo(std::string *info) {
   std::ostringstream string_stream;
   string_stream << "# Stats\r\n";
@@ -1078,7 +1077,7 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
 
     std::lock_guard<std::mutex> lg(db_job_mu_);
     string_stream << "bgsave_in_progress:" << (is_bgsave_in_progress_ ? 1 : 0) << "\r\n";
-    string_stream << "last_bgsave_time:" << last_bgsave_time_ << "\r\n";
+    string_stream << "last_bgsave_time:" << (last_bgsave_time_ == -1 ? start_time_ : last_bgsave_time_) << "\r\n";
     string_stream << "last_bgsave_status:" << last_bgsave_status_ << "\r\n";
     string_stream << "last_bgsave_time_sec:" << last_bgsave_time_sec_ << "\r\n";
   }
@@ -1252,7 +1251,11 @@ Status Server::AsyncCompactDB(const std::string &begin_key, const std::string &e
     std::unique_ptr<Slice> begin = nullptr, end = nullptr;
     if (!begin_key.empty()) begin = std::make_unique<Slice>(begin_key);
     if (!end_key.empty()) end = std::make_unique<Slice>(end_key);
-    storage->Compact(begin.get(), end.get());
+
+    auto s = storage->Compact(begin.get(), end.get());
+    if (!s.ok()) {
+      LOG(ERROR) << "[task runner] Failed to do compaction: " << s.ToString();
+    }
 
     std::lock_guard<std::mutex> lg(db_job_mu_);
     db_compacting_ = false;
@@ -1274,9 +1277,9 @@ Status Server::AsyncBgSaveDB() {
 
     std::lock_guard<std::mutex> lg(db_job_mu_);
     is_bgsave_in_progress_ = false;
-    last_bgsave_time_ = static_cast<int>(start_bgsave_time);
+    last_bgsave_time_ = start_bgsave_time;
     last_bgsave_status_ = s.IsOK() ? "ok" : "err";
-    last_bgsave_time_sec_ = static_cast<int>(stop_bgsave_time - start_bgsave_time);
+    last_bgsave_time_sec_ = stop_bgsave_time - start_bgsave_time;
   });
 }
 
@@ -1301,8 +1304,12 @@ Status Server::AsyncScanDBSize(const std::string &ns) {
 
   return task_runner_.TryPublish([ns, this] {
     redis::Database db(storage, ns);
+
     KeyNumStats stats;
-    db.GetKeyNumStats("", &stats);
+    auto s = db.GetKeyNumStats("", &stats);
+    if (!s.ok()) {
+      LOG(ERROR) << "failed to retrieve key num stats: " << s.ToString();
+    }
 
     std::lock_guard<std::mutex> lg(db_job_mu_);
 
@@ -1408,7 +1415,8 @@ time_t Server::GetLastScanTime(const std::string &ns) {
   return 0;
 }
 
-void Server::SlowlogPushEntryIfNeeded(const std::vector<std::string> *args, uint64_t duration) {
+void Server::SlowlogPushEntryIfNeeded(const std::vector<std::string> *args, uint64_t duration,
+                                      const redis::Connection *conn) {
   int64_t threshold = config_->slowlog_log_slower_than;
   if (threshold < 0 || static_cast<int64_t>(duration) < threshold) return;
 
@@ -1429,6 +1437,9 @@ void Server::SlowlogPushEntryIfNeeded(const std::vector<std::string> *args, uint
   }
 
   entry->duration = duration;
+  entry->client_name = conn->GetName();
+  entry->ip = conn->GetIP();
+  entry->port = conn->GetPort();
   slow_log_.PushEntry(std::move(entry));
 }
 
@@ -1507,11 +1518,9 @@ Status Server::LookupAndCreateCommand(const std::string &cmd_name, std::unique_p
 }
 
 Status Server::ScriptExists(const std::string &sha) {
-  lua_getglobal(lua_, (REDIS_LUA_FUNC_SHA_PREFIX + sha).c_str());
-  if (!lua_isnil(lua_, -1)) {
+  if (lua::ScriptExists(lua_, sha)) {
     return Status::OK();
   }
-  lua_pop(lua_, 1);
 
   std::string body;
   return ScriptGet(sha, &body);
@@ -1537,10 +1546,12 @@ void Server::ScriptReset() {
   lua::DestroyState(lua);
 }
 
-void Server::ScriptFlush() {
+Status Server::ScriptFlush() {
   auto cf = storage->GetCFHandle(engine::kPropagateColumnFamilyName);
-  storage->FlushScripts(storage->DefaultWriteOptions(), cf);
+  auto s = storage->FlushScripts(storage->DefaultWriteOptions(), cf);
+  if (!s.ok()) return {Status::NotOK, s.ToString()};
   ScriptReset();
+  return Status::OK();
 }
 
 // Generally, we store data into RocksDB and just replicate WAL instead of propagating
@@ -1634,7 +1645,7 @@ void Server::AdjustOpenFilesLimit() {
   }
 }
 
-std::string ServerLogData::Encode() {
+std::string ServerLogData::Encode() const {
   if (type_ == kReplIdLog) {
     return std::string(1, kReplIdTag) + " " + content_;
   }
@@ -1680,15 +1691,24 @@ void Server::updateAllWatchedKeys() {
 }
 
 void Server::UpdateWatchedKeysFromArgs(const std::vector<std::string> &args, const redis::CommandAttributes &attr) {
-  if (attr.IsWrite() && watched_key_size_ > 0) {
+  if ((attr.flags & redis::kCmdWrite) && watched_key_size_ > 0) {
     if (attr.key_range.first_key > 0) {
       updateWatchedKeysFromRange(args, attr.key_range);
-    } else if (attr.key_range.first_key < 0) {
+    } else if (attr.key_range.first_key == -1) {
       redis::CommandKeyRange range = attr.key_range_gen(args);
 
       if (range.first_key > 0) {
         updateWatchedKeysFromRange(args, range);
       }
+    } else if (attr.key_range.first_key == -2) {
+      std::vector<redis::CommandKeyRange> vec_range = attr.key_range_vec_gen(args);
+
+      for (const auto &range : vec_range) {
+        if (range.first_key > 0) {
+          updateWatchedKeysFromRange(args, range);
+        }
+      }
+
     } else {
       // support commands like flushdb (write flag && key range {0,0,0})
       updateAllWatchedKeys();
