@@ -52,13 +52,14 @@ rocksdb::Status WriteBatchExtractor::PutCF(uint32_t column_family_id, const Slic
   std::vector<std::string> command_args;
 
   if (column_family_id == kColumnFamilyIDMetadata) {
-    ExtractNamespaceKey(key, &ns, &user_key, is_slot_id_encoded_);
+    std::tie(ns, user_key) = ExtractNamespaceKey<std::string>(key, is_slot_id_encoded_);
     if (slot_id_ >= 0 && static_cast<uint16_t>(slot_id_) != GetSlotIdFromKey(user_key)) {
       return rocksdb::Status::OK();
     }
 
     Metadata metadata(kRedisNone);
-    metadata.Decode(value.ToString());
+    auto s = metadata.Decode(value);
+    if (!s.ok()) return s;
 
     if (metadata.Type() == kRedisString) {
       command_args = {"SET", user_key, value.ToString().substr(Metadata::GetOffsetAfterExpire(value[0]))};
@@ -92,7 +93,7 @@ rocksdb::Status WriteBatchExtractor::PutCF(uint32_t column_family_id, const Slic
       }
 
       StreamMetadata stream_metadata;
-      auto s = stream_metadata.Decode(value.ToString());
+      auto s = stream_metadata.Decode(value);
       if (!s.ok()) return s;
 
       command_args = {"XSETID",
@@ -244,6 +245,7 @@ rocksdb::Status WriteBatchExtractor::PutCF(uint32_t column_family_id, const Slic
         }
         break;
       }
+        // TODO: to implement the case of kRedisBloomFilter
       default:
         break;
     }
@@ -272,7 +274,7 @@ rocksdb::Status WriteBatchExtractor::DeleteCF(uint32_t column_family_id, const S
 
   if (column_family_id == kColumnFamilyIDMetadata) {
     std::string user_key;
-    ExtractNamespaceKey(key, &ns, &user_key, is_slot_id_encoded_);
+    std::tie(ns, user_key) = ExtractNamespaceKey<std::string>(key, is_slot_id_encoded_);
 
     if (slot_id_ >= 0 && static_cast<uint16_t>(slot_id_) != GetSlotIdFromKey(user_key)) {
       return rocksdb::Status::OK();
