@@ -44,20 +44,25 @@ rocksdb::Status Database::GetMetadata(RedisType type, const Slice &ns_key, Metad
   std::string bytes;
   auto s = GetRawMetadata(ns_key, &bytes);
   if (!s.ok()) return s;
-  metadata->Decode(bytes);
+  s = metadata->Decode(bytes);
+  if (!s.ok()) return s;
 
   if (metadata->Expired()) {
-    metadata->Decode(old_metadata);
+    // error discarded here since it already failed
+    auto _ [[maybe_unused]] = metadata->Decode(old_metadata);
     return rocksdb::Status::NotFound(kErrMsgKeyExpired);
   }
   if (metadata->Type() != type &&
       (metadata->size > 0 || metadata->Type() == kRedisString || metadata->Type() == kRedisStream)) {
-    metadata->Decode(old_metadata);
+    // error discarded here since it already failed
+    auto _ [[maybe_unused]] = metadata->Decode(old_metadata);
     return rocksdb::Status::InvalidArgument(kErrMsgWrongType);
   }
-  if (metadata->size == 0 && type != kRedisStream) {  // stream is allowed to be empty
-    metadata->Decode(old_metadata);
-    return rocksdb::Status::NotFound("no elements");
+  if (metadata->size == 0 && type != kRedisStream &&
+      type != kRedisBloomFilter) {  // stream and bloom is allowed to be empty
+    // error discarded here since it already failed
+    auto _ [[maybe_unused]] = metadata->Decode(old_metadata);
+    return rocksdb::Status::NotFound("no element found");
   }
   return s;
 }
@@ -70,21 +75,20 @@ rocksdb::Status Database::GetRawMetadata(const Slice &ns_key, std::string *bytes
 }
 
 rocksdb::Status Database::GetRawMetadataByUserKey(const Slice &user_key, std::string *bytes) {
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
-  return GetRawMetadata(ns_key, bytes);
+  return GetRawMetadata(AppendNamespacePrefix(user_key), bytes);
 }
 
 rocksdb::Status Database::Expire(const Slice &user_key, uint64_t timestamp) {
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
 
   std::string value;
   Metadata metadata(kRedisNone, false);
   LockGuard guard(storage_->GetLockManager(), ns_key);
   rocksdb::Status s = storage_->Get(rocksdb::ReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s;
-  metadata.Decode(value);
+
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
   if (metadata.Expired()) {
     return rocksdb::Status::NotFound(kErrMsgKeyExpired);
   }
@@ -108,21 +112,68 @@ rocksdb::Status Database::Expire(const Slice &user_key, uint64_t timestamp) {
 }
 
 rocksdb::Status Database::Del(const Slice &user_key) {
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
 
   std::string value;
   LockGuard guard(storage_->GetLockManager(), ns_key);
   rocksdb::Status s = storage_->Get(rocksdb::ReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s;
   Metadata metadata(kRedisNone, false);
-  metadata.Decode(value);
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
   if (metadata.Expired()) {
     return rocksdb::Status::NotFound(kErrMsgKeyExpired);
   }
   return storage_->Delete(storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
 }
 
+rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted_cnt) {
+  *deleted_cnt = 0;
+
+  std::vector<std::string> lock_keys;
+  lock_keys.reserve(keys.size());
+  for (const auto &key : keys) {
+    std::string ns_key = AppendNamespacePrefix(key);
+    lock_keys.emplace_back(std::move(ns_key));
+  }
+  MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
+
+  auto batch = storage_->GetWriteBatchBase();
+  WriteBatchLogData log_data(kRedisNone);
+  batch->PutLogData(log_data.Encode());
+
+  std::vector<Slice> slice_keys;
+  slice_keys.reserve(lock_keys.size());
+  for (const auto &ns_key : lock_keys) {
+    slice_keys.emplace_back(ns_key);
+  }
+
+  LatestSnapShot ss(storage_);
+  rocksdb::ReadOptions read_options = storage_->DefaultMultiGetOptions();
+  read_options.snapshot = ss.GetSnapShot();
+  std::vector<rocksdb::Status> statuses(slice_keys.size());
+  std::vector<rocksdb::PinnableSlice> pin_values(slice_keys.size());
+  storage_->MultiGet(read_options, metadata_cf_handle_, slice_keys.size(), slice_keys.data(), pin_values.data(),
+                     statuses.data());
+
+  for (size_t i = 0; i < slice_keys.size(); i++) {
+    if (!statuses[i].ok() && !statuses[i].IsNotFound()) return statuses[i];
+    if (statuses[i].IsNotFound()) continue;
+
+    Metadata metadata(kRedisNone, false);
+    auto s = metadata.Decode(pin_values[i]);
+    if (!s.ok()) continue;
+    if (metadata.Expired()) continue;
+
+    batch->Delete(metadata_cf_handle_, lock_keys[i]);
+    *deleted_cnt += 1;
+  }
+
+  if (*deleted_cnt == 0) return rocksdb::Status::OK();
+
+  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+}
+
 rocksdb::Status Database::Exists(const std::vector<Slice> &keys, int *ret) {
   *ret = 0;
   LatestSnapShot ss(storage_);
@@ -130,13 +181,15 @@ rocksdb::Status Database::Exists(const std::vector<Slice> &keys, int *ret) {
   read_options.snapshot = ss.GetSnapShot();
 
   rocksdb::Status s;
-  std::string ns_key, value;
+  std::string value;
   for (const auto &key : keys) {
-    AppendNamespacePrefix(key, &ns_key);
+    std::string ns_key = AppendNamespacePrefix(key);
     s = storage_->Get(read_options, metadata_cf_handle_, ns_key, &value);
+    if (!s.ok() && !s.IsNotFound()) return s;
     if (s.ok()) {
       Metadata metadata(kRedisNone, false);
-      metadata.Decode(value);
+      s = metadata.Decode(value);
+      if (!s.ok()) return s;
       if (!metadata.Expired()) *ret += 1;
     }
   }
@@ -144,8 +197,7 @@ rocksdb::Status Database::Exists(const std::vector<Slice> &keys, int *ret) {
 }
 
 rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
 
   *ttl = -2;  // ttl is -2 when the key does not exist or expired
   LatestSnapShot ss(storage_);
@@ -156,34 +208,36 @@ rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
-  metadata.Decode(value);
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
   *ttl = metadata.TTL();
 
   return rocksdb::Status::OK();
 }
 
-void Database::GetKeyNumStats(const std::string &prefix, KeyNumStats *stats) { Keys(prefix, nullptr, stats); }
+rocksdb::Status Database::GetKeyNumStats(const std::string &prefix, KeyNumStats *stats) {
+  return Keys(prefix, nullptr, stats);
+}
 
-void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, KeyNumStats *stats) {
+rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::string> *keys, KeyNumStats *stats) {
   uint16_t slot_id = 0;
-  std::string ns_prefix, ns, user_key, value;
+  std::string ns_prefix;
   if (namespace_ != kDefaultNamespace || keys != nullptr) {
     if (storage_->IsSlotIdEncoded()) {
-      ComposeNamespaceKey(namespace_, "", &ns_prefix, false);
+      ns_prefix = ComposeNamespaceKey(namespace_, "", false);
       if (!prefix.empty()) {
         PutFixed16(&ns_prefix, slot_id);
         ns_prefix.append(prefix);
       }
     } else {
-      AppendNamespacePrefix(prefix, &ns_prefix);
+      ns_prefix = AppendNamespacePrefix(prefix);
     }
   }
 
   uint64_t ttl_sum = 0;
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
   auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
 
   while (true) {
@@ -193,8 +247,8 @@ void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, K
         break;
       }
       Metadata metadata(kRedisNone, false);
-      value = iter->value().ToString();
-      metadata.Decode(value);
+      auto s = metadata.Decode(iter->value());
+      if (!s.ok()) continue;
       if (metadata.Expired()) {
         if (stats) stats->n_expired++;
         continue;
@@ -208,8 +262,8 @@ void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, K
         }
       }
       if (keys) {
-        ExtractNamespaceKey(iter->key(), &ns, &user_key, storage_->IsSlotIdEncoded());
-        keys->emplace_back(user_key);
+        auto [_, user_key] = ExtractNamespaceKey(iter->key(), storage_->IsSlotIdEncoded());
+        keys->emplace_back(user_key.ToString());
       }
     }
 
@@ -217,7 +271,7 @@ void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, K
     if (prefix.empty()) break;
     if (++slot_id >= HASH_SLOTS_SIZE) break;
 
-    ComposeNamespaceKey(namespace_, "", &ns_prefix, false);
+    ns_prefix = ComposeNamespaceKey(namespace_, "", false);
     PutFixed16(&ns_prefix, slot_id);
     ns_prefix.append(prefix);
   }
@@ -225,6 +279,8 @@ void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, K
   if (stats && stats->n_expires > 0) {
     stats->avg_ttl = ttl_sum / stats->n_expires / 1000;
   }
+
+  return rocksdb::Status::OK();
 }
 
 rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const std::string &prefix,
@@ -232,24 +288,24 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
   end_cursor->clear();
   uint64_t cnt = 0;
   uint16_t slot_start = 0;
-  std::string ns_prefix, ns_cursor, ns, user_key, value, index_key;
+  std::string ns_prefix;
+  std::string user_key;
 
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
   auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
 
-  AppendNamespacePrefix(cursor, &ns_cursor);
+  std::string ns_cursor = AppendNamespacePrefix(cursor);
   if (storage_->IsSlotIdEncoded()) {
     slot_start = cursor.empty() ? 0 : GetSlotIdFromKey(cursor);
-    ComposeNamespaceKey(namespace_, "", &ns_prefix, false);
+    ns_prefix = ComposeNamespaceKey(namespace_, "", false);
     if (!prefix.empty()) {
       PutFixed16(&ns_prefix, slot_start);
       ns_prefix.append(prefix);
     }
   } else {
-    AppendNamespacePrefix(prefix, &ns_prefix);
+    ns_prefix = AppendNamespacePrefix(prefix);
   }
 
   if (!cursor.empty()) {
@@ -270,10 +326,11 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
         break;
       }
       Metadata metadata(kRedisNone, false);
-      value = iter->value().ToString();
-      metadata.Decode(value);
+      auto s = metadata.Decode(iter->value());
+      if (!s.ok()) continue;
+
       if (metadata.Expired()) continue;
-      ExtractNamespaceKey(iter->key(), &ns, &user_key, storage_->IsSlotIdEncoded());
+      std::tie(std::ignore, user_key) = ExtractNamespaceKey<std::string>(iter->key(), storage_->IsSlotIdEncoded());
       keys->emplace_back(user_key);
       cnt++;
     }
@@ -296,7 +353,7 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
     if (slot_id > slot_start + HASH_SLOTS_MAX_ITERATIONS) {
       if (keys->empty()) {
         if (iter->Valid()) {
-          ExtractNamespaceKey(iter->key(), &ns, &user_key, storage_->IsSlotIdEncoded());
+          std::tie(std::ignore, user_key) = ExtractNamespaceKey<std::string>(iter->key(), storage_->IsSlotIdEncoded());
           auto res = std::mismatch(prefix.begin(), prefix.end(), user_key.begin());
           if (res.first == prefix.end()) {
             keys->emplace_back(user_key);
@@ -310,7 +367,7 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
       break;
     }
 
-    ComposeNamespaceKey(namespace_, "", &ns_prefix, false);
+    ns_prefix = ComposeNamespaceKey(namespace_, "", false);
     PutFixed16(&ns_prefix, slot_id);
     ns_prefix.append(prefix);
     iter->Seek(ns_prefix);
@@ -342,8 +399,8 @@ rocksdb::Status Database::RandomKey(const std::string &cursor, std::string *key)
 }
 
 rocksdb::Status Database::FlushDB() {
-  std::string prefix, begin_key, end_key;
-  ComposeNamespaceKey(namespace_, "", &prefix, false);
+  std::string begin_key, end_key;
+  std::string prefix = ComposeNamespaceKey(namespace_, "", false);
   auto s = FindKeyRangeWithPrefix(prefix, std::string(), &begin_key, &end_key);
   if (!s.ok()) {
     return rocksdb::Status::OK();
@@ -358,9 +415,8 @@ rocksdb::Status Database::FlushDB() {
 
 rocksdb::Status Database::FlushAll() {
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
   auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
   iter->SeekToFirst();
   if (!iter->Valid()) {
@@ -382,8 +438,7 @@ rocksdb::Status Database::FlushAll() {
 rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *infos) {
   infos->clear();
 
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
 
   LatestSnapShot ss(storage_);
   rocksdb::ReadOptions read_options;
@@ -393,7 +448,8 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
-  metadata.Decode(value);
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
 
   infos->emplace_back("namespace");
   infos->emplace_back(namespace_);
@@ -422,7 +478,7 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
 
   if (metadata.Type() == kRedisList) {
     ListMetadata list_metadata(false);
-    GetMetadata(kRedisList, ns_key, &list_metadata);
+    s = GetMetadata(kRedisList, ns_key, &list_metadata);
     if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
     infos->emplace_back("head");
     infos->emplace_back(std::to_string(list_metadata.head));
@@ -434,8 +490,7 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
 }
 
 rocksdb::Status Database::Type(const Slice &user_key, RedisType *type) {
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
 
   *type = kRedisNone;
   LatestSnapShot ss(storage_);
@@ -446,7 +501,8 @@ rocksdb::Status Database::Type(const Slice &user_key, RedisType *type) {
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
-  metadata.Decode(value);
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
   if (metadata.Expired()) {
     *type = kRedisNone;
   } else {
@@ -455,8 +511,8 @@ rocksdb::Status Database::Type(const Slice &user_key, RedisType *type) {
   return rocksdb::Status::OK();
 }
 
-void Database::AppendNamespacePrefix(const Slice &user_key, std::string *output) {
-  ComposeNamespaceKey(namespace_, user_key, output, storage_->IsSlotIdEncoded());
+std::string Database::AppendNamespacePrefix(const Slice &user_key) {
+  return ComposeNamespaceKey(namespace_, user_key, storage_->IsSlotIdEncoded());
 }
 
 rocksdb::Status Database::FindKeyRangeWithPrefix(const std::string &prefix, const std::string &prefix_end,
@@ -472,9 +528,8 @@ rocksdb::Status Database::FindKeyRangeWithPrefix(const std::string &prefix, cons
   end->clear();
 
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
   auto iter = util::UniqueIterator(storage_, read_options, cf_handle);
   iter->Seek(prefix);
   if (!iter->Valid() || !iter->key().starts_with(prefix)) {
@@ -513,9 +568,8 @@ rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
     return rocksdb::Status::Aborted("It is not in cluster mode");
   }
 
-  std::string prefix, prefix_end;
-  ComposeSlotKeyPrefix(ns, slot, &prefix);
-  ComposeSlotKeyPrefix(ns, slot + 1, &prefix_end);
+  std::string prefix = ComposeSlotKeyPrefix(ns, slot);
+  std::string prefix_end = ComposeSlotKeyPrefix(ns, slot + 1);
   auto s = storage_->DeleteRange(prefix, prefix_end);
   if (!s.ok()) {
     return s;
@@ -526,15 +580,13 @@ rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
 rocksdb::Status Database::GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slotskeys, std::vector<std::string> *keys,
                                           int count) {
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
 
   auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
   bool end = false;
   for (int i = 0; i < HASH_SLOTS_SIZE; i++) {
-    std::string prefix;
-    ComposeSlotKeyPrefix(namespace_, i, &prefix);
+    std::string prefix = ComposeSlotKeyPrefix(namespace_, i);
     uint64_t total = 0;
     int cnt = 0;
     if (slot != -1 && i != slot) {
@@ -549,9 +601,8 @@ rocksdb::Status Database::GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slo
       if (slot != -1 && count > 0 && !end) {
         // Get user key
         if (cnt < count) {
-          std::string ns, user_key;
-          ExtractNamespaceKey(iter->key(), &ns, &user_key, true);
-          keys->push_back(user_key);
+          auto [_, user_key] = ExtractNamespaceKey(iter->key(), true);
+          keys->emplace_back(user_key.ToString());
           cnt++;
         }
       }
@@ -567,27 +618,21 @@ rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const
                                     const std::string &subkey_prefix, std::vector<std::string> *keys,
                                     std::vector<std::string> *values) {
   uint64_t cnt = 0;
-  std::string ns_key;
-  AppendNamespacePrefix(user_key, &ns_key);
+  std::string ns_key = AppendNamespacePrefix(user_key);
   Metadata metadata(type, false);
   rocksdb::Status s = GetMetadata(type, ns_key, &metadata);
   if (!s.ok()) return s;
 
   LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
   read_options.snapshot = ss.GetSnapShot();
-  storage_->SetReadOptions(read_options);
   auto iter = util::UniqueIterator(storage_, read_options);
-  std::string match_prefix_key;
-  if (!subkey_prefix.empty()) {
-    InternalKey(ns_key, subkey_prefix, metadata.version, storage_->IsSlotIdEncoded()).Encode(&match_prefix_key);
-  } else {
-    InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode(&match_prefix_key);
-  }
+  std::string match_prefix_key =
+      InternalKey(ns_key, subkey_prefix, metadata.version, storage_->IsSlotIdEncoded()).Encode();
 
   std::string start_key;
   if (!cursor.empty()) {
-    InternalKey(ns_key, cursor, metadata.version, storage_->IsSlotIdEncoded()).Encode(&start_key);
+    start_key = InternalKey(ns_key, cursor, metadata.version, storage_->IsSlotIdEncoded()).Encode();
   } else {
     start_key = match_prefix_key;
   }
@@ -613,11 +658,11 @@ rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const
   return rocksdb::Status::OK();
 }
 
-RedisType WriteBatchLogData::GetRedisType() { return type_; }
+RedisType WriteBatchLogData::GetRedisType() const { return type_; }
 
 std::vector<std::string> *WriteBatchLogData::GetArguments() { return &args_; }
 
-std::string WriteBatchLogData::Encode() {
+std::string WriteBatchLogData::Encode() const {
   std::string ret = std::to_string(type_);
   for (const auto &arg : args_) {
     ret += " " + arg;
