@@ -34,28 +34,31 @@ class Database {
   static constexpr uint64_t RANDOM_KEY_SCAN_LIMIT = 60;
 
   explicit Database(engine::Storage *storage, std::string ns = "");
-  rocksdb::Status GetMetadata(RedisType type, const Slice &ns_key, Metadata *metadata);
-  rocksdb::Status GetRawMetadata(const Slice &ns_key, std::string *bytes);
-  rocksdb::Status GetRawMetadataByUserKey(const Slice &user_key, std::string *bytes);
-  rocksdb::Status Expire(const Slice &user_key, uint64_t timestamp);
-  rocksdb::Status Del(const Slice &user_key);
-  rocksdb::Status Exists(const std::vector<Slice> &keys, int *ret);
-  rocksdb::Status TTL(const Slice &user_key, int64_t *ttl);
-  rocksdb::Status Type(const Slice &user_key, RedisType *type);
-  rocksdb::Status Dump(const Slice &user_key, std::vector<std::string> *infos);
-  rocksdb::Status FlushDB();
-  rocksdb::Status FlushAll();
-  void GetKeyNumStats(const std::string &prefix, KeyNumStats *stats);
-  void Keys(const std::string &prefix, std::vector<std::string> *keys = nullptr, KeyNumStats *stats = nullptr);
-  rocksdb::Status Scan(const std::string &cursor, uint64_t limit, const std::string &prefix,
-                       std::vector<std::string> *keys, std::string *end_cursor = nullptr);
-  rocksdb::Status RandomKey(const std::string &cursor, std::string *key);
-  void AppendNamespacePrefix(const Slice &user_key, std::string *output);
-  rocksdb::Status FindKeyRangeWithPrefix(const std::string &prefix, const std::string &prefix_end, std::string *begin,
-                                         std::string *end, rocksdb::ColumnFamilyHandle *cf_handle = nullptr);
-  rocksdb::Status ClearKeysOfSlot(const rocksdb::Slice &ns, int slot);
-  rocksdb::Status GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slotskeys, std::vector<std::string> *keys,
-                                  int count);
+  [[nodiscard]] rocksdb::Status GetMetadata(RedisType type, const Slice &ns_key, Metadata *metadata);
+  [[nodiscard]] rocksdb::Status GetRawMetadata(const Slice &ns_key, std::string *bytes);
+  [[nodiscard]] rocksdb::Status GetRawMetadataByUserKey(const Slice &user_key, std::string *bytes);
+  [[nodiscard]] rocksdb::Status Expire(const Slice &user_key, uint64_t timestamp);
+  [[nodiscard]] rocksdb::Status Del(const Slice &user_key);
+  [[nodiscard]] rocksdb::Status MDel(const std::vector<Slice> &keys, uint64_t *deleted_cnt);
+  [[nodiscard]] rocksdb::Status Exists(const std::vector<Slice> &keys, int *ret);
+  [[nodiscard]] rocksdb::Status TTL(const Slice &user_key, int64_t *ttl);
+  [[nodiscard]] rocksdb::Status Type(const Slice &user_key, RedisType *type);
+  [[nodiscard]] rocksdb::Status Dump(const Slice &user_key, std::vector<std::string> *infos);
+  [[nodiscard]] rocksdb::Status FlushDB();
+  [[nodiscard]] rocksdb::Status FlushAll();
+  [[nodiscard]] rocksdb::Status GetKeyNumStats(const std::string &prefix, KeyNumStats *stats);
+  [[nodiscard]] rocksdb::Status Keys(const std::string &prefix, std::vector<std::string> *keys = nullptr,
+                                     KeyNumStats *stats = nullptr);
+  [[nodiscard]] rocksdb::Status Scan(const std::string &cursor, uint64_t limit, const std::string &prefix,
+                                     std::vector<std::string> *keys, std::string *end_cursor = nullptr);
+  [[nodiscard]] rocksdb::Status RandomKey(const std::string &cursor, std::string *key);
+  std::string AppendNamespacePrefix(const Slice &user_key);
+  [[nodiscard]] rocksdb::Status FindKeyRangeWithPrefix(const std::string &prefix, const std::string &prefix_end,
+                                                       std::string *begin, std::string *end,
+                                                       rocksdb::ColumnFamilyHandle *cf_handle = nullptr);
+  [[nodiscard]] rocksdb::Status ClearKeysOfSlot(const rocksdb::Slice &ns, int slot);
+  [[nodiscard]] rocksdb::Status GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slotskeys,
+                                                std::vector<std::string> *keys, int count);
 
  protected:
   engine::Storage *storage_;
@@ -68,7 +71,7 @@ class Database {
     explicit LatestSnapShot(engine::Storage *storage)
         : storage_(storage), snapshot_(storage_->GetDB()->GetSnapshot()) {}
     ~LatestSnapShot() { storage_->GetDB()->ReleaseSnapshot(snapshot_); }
-    const rocksdb::Snapshot *GetSnapShot() { return snapshot_; }
+    const rocksdb::Snapshot *GetSnapShot() const { return snapshot_; }
 
     LatestSnapShot(const LatestSnapShot &) = delete;
     LatestSnapShot &operator=(const LatestSnapShot &) = delete;
@@ -93,9 +96,9 @@ class WriteBatchLogData {
   explicit WriteBatchLogData(RedisType type) : type_(type) {}
   explicit WriteBatchLogData(RedisType type, std::vector<std::string> &&args) : type_(type), args_(std::move(args)) {}
 
-  RedisType GetRedisType();
+  RedisType GetRedisType() const;
   std::vector<std::string> *GetArguments();
-  std::string Encode();
+  std::string Encode() const;
   Status Decode(const rocksdb::Slice &blob);
 
  private:
