@@ -33,6 +33,7 @@
 #include "lua.h"
 #include "parse_util.h"
 #include "rand.h"
+#include "scope_exit.h"
 #include "server/redis_connection.h"
 #include "server/server.h"
 #include "sha1.h"
@@ -292,6 +293,12 @@ Status EvalGenericCommand(redis::Connection *conn, const std::string &body_or_sh
   return Status::OK();
 }
 
+bool ScriptExists(lua_State *lua, const std::string &sha) {
+  lua_getglobal(lua, (REDIS_LUA_FUNC_SHA_PREFIX + sha).c_str());
+  auto exit = MakeScopeExit([lua] { lua_pop(lua, 1); });
+  return !lua_isnil(lua, -1);
+}
+
 int RedisCallCommand(lua_State *lua) { return RedisGenericCommand(lua, 1); }
 
 int RedisPCallCommand(lua_State *lua) { return RedisGenericCommand(lua, 0); }
@@ -349,7 +356,8 @@ int RedisGenericCommand(lua_State *lua, int raise_error) {
     return raise_error ? RaiseError(lua) : 1;
   }
   auto attributes = cmd->GetAttributes();
-  if (attributes->flags & redis::kCmdNoScript) {
+  auto cmd_flags = attributes->GenerateFlags(args);
+  if (cmd_flags & redis::kCmdNoScript) {
     PushError(lua, "This Redis command is not allowed from scripts");
     return raise_error ? RaiseError(lua) : 1;
   }
@@ -371,7 +379,7 @@ int RedisGenericCommand(lua_State *lua, int raise_error) {
     }
   }
 
-  if (config->slave_readonly && srv->IsSlave() && attributes->IsWrite()) {
+  if (config->slave_readonly && srv->IsSlave() && (cmd_flags & redis::kCmdWrite)) {
     PushError(lua, "READONLY You can't write against a read only slave.");
     return raise_error ? RaiseError(lua) : 1;
   }
@@ -398,7 +406,7 @@ int RedisGenericCommand(lua_State *lua, int raise_error) {
   auto end = std::chrono::high_resolution_clock::now();
   uint64_t duration = std::chrono::duration_cast<std::chrono::microseconds>(end - start).count();
   if (is_profiling) conn->RecordProfilingSampleIfNeed(cmd_name, duration);
-  srv->SlowlogPushEntryIfNeeded(&args, duration);
+  srv->SlowlogPushEntryIfNeeded(&args, duration, conn);
   srv->stats.IncrLatency(static_cast<uint64_t>(duration), cmd_name);
   srv->FeedMonitorConns(conn, args);
   if (!s) {
