@@ -86,9 +86,19 @@ void Storage::SetWriteOptions(const Config::RocksDB::WriteOptions &config) {
   write_opts_.memtable_insert_hint_per_batch = config.memtable_insert_hint_per_batch;
 }
 
-void Storage::SetReadOptions(rocksdb::ReadOptions &read_options) {
+rocksdb::ReadOptions Storage::DefaultScanOptions() const {
+  rocksdb::ReadOptions read_options;
   read_options.fill_cache = false;
   read_options.async_io = config_->rocks_db.read_options.async_io;
+
+  return read_options;
+}
+
+rocksdb::ReadOptions Storage::DefaultMultiGetOptions() const {
+  rocksdb::ReadOptions read_options;
+  read_options.async_io = config_->rocks_db.read_options.async_io;
+
+  return read_options;
 }
 
 rocksdb::BlockBasedTableOptions Storage::InitTableOptions() {
@@ -235,23 +245,24 @@ Status Storage::Open(bool read_only) {
   db_closing_ = false;
 
   bool cache_index_and_filter_blocks = config_->rocks_db.cache_index_and_filter_blocks;
+  size_t block_cache_size = config_->rocks_db.block_cache_size * MiB;
   size_t metadata_block_cache_size = config_->rocks_db.metadata_block_cache_size * MiB;
   size_t subkey_block_cache_size = config_->rocks_db.subkey_block_cache_size * MiB;
+  if (block_cache_size == 0) {
+    block_cache_size = metadata_block_cache_size + subkey_block_cache_size;
+  }
 
   rocksdb::Options options = InitRocksDBOptions();
-  if (auto s = CreateColumnFamilies(options); !s.IsOK()) {
-    return s.Prefixed("failed to create column families");
+  if (!read_only) {
+    if (auto s = CreateColumnFamilies(options); !s.IsOK()) {
+      return s.Prefixed("failed to create column families");
+    }
   }
 
-  std::shared_ptr<rocksdb::Cache> shared_block_cache;
-  if (config_->rocks_db.share_metadata_and_subkey_block_cache) {
-    size_t shared_block_cache_size = metadata_block_cache_size + subkey_block_cache_size;
-    shared_block_cache = rocksdb::NewLRUCache(shared_block_cache_size, -1, false, 0.75);
-  }
+  std::shared_ptr<rocksdb::Cache> shared_block_cache = rocksdb::NewLRUCache(block_cache_size, -1, false, 0.75);
 
   rocksdb::BlockBasedTableOptions metadata_table_opts = InitTableOptions();
-  metadata_table_opts.block_cache =
-      shared_block_cache ? shared_block_cache : rocksdb::NewLRUCache(metadata_block_cache_size, -1, false, 0.75);
+  metadata_table_opts.block_cache = shared_block_cache;
   metadata_table_opts.pin_l0_filter_and_index_blocks_in_cache = true;
   metadata_table_opts.cache_index_and_filter_blocks = cache_index_and_filter_blocks;
   metadata_table_opts.cache_index_and_filter_blocks_with_high_priority = true;
@@ -268,8 +279,7 @@ Status Storage::Open(bool read_only) {
   SetBlobDB(&metadata_opts);
 
   rocksdb::BlockBasedTableOptions subkey_table_opts = InitTableOptions();
-  subkey_table_opts.block_cache =
-      shared_block_cache ? shared_block_cache : rocksdb::NewLRUCache(subkey_block_cache_size, -1, false, 0.75);
+  subkey_table_opts.block_cache = shared_block_cache;
   subkey_table_opts.pin_l0_filter_and_index_blocks_in_cache = true;
   subkey_table_opts.cache_index_and_filter_blocks = cache_index_and_filter_blocks;
   subkey_table_opts.cache_index_and_filter_blocks_with_high_priority = true;
@@ -633,6 +643,9 @@ rocksdb::ColumnFamilyHandle *Storage::GetCFHandle(const std::string &name) {
 rocksdb::Status Storage::Compact(const Slice *begin, const Slice *end) {
   rocksdb::CompactRangeOptions compact_opts;
   compact_opts.change_level = true;
+  // For the manual compaction, we would like to force the bottommost level to be compacted.
+  // Or it may use the trivial mode and some expired key-values were still exist in the bottommost level.
+  compact_opts.bottommost_level_compaction = rocksdb::BottommostLevelCompaction::kForceOptimized;
   for (const auto &cf_handle : cf_handles_) {
     rocksdb::Status s = db_->CompactRange(compact_opts, cf_handle, begin, end);
     if (!s.ok()) return s;
@@ -645,8 +658,8 @@ uint64_t Storage::GetTotalSize(const std::string &ns) {
     return sst_file_manager_->GetTotalSize();
   }
 
-  std::string prefix, begin_key, end_key;
-  ComposeNamespaceKey(ns, "", &prefix, false);
+  std::string begin_key, end_key;
+  std::string prefix = ComposeNamespaceKey(ns, "", false);
 
   redis::Database db(this, ns);
   uint64_t size = 0, total_size = 0;
@@ -1094,7 +1107,7 @@ bool Storage::ReplDataManager::FileExists(Storage *storage, const std::string &d
 
     if (slice.size() == 0) return false;
 
-    tmp_crc = rocksdb::crc32c::Extend(0, slice.ToString().c_str(), slice.size());
+    tmp_crc = rocksdb::crc32c::Extend(0, slice.data(), slice.size());
     size -= slice.size();
   }
 
