@@ -885,4 +885,296 @@ func TestList(t *testing.T) {
 			})
 		}
 	}
+
+	t.Run("Test BLMOVE on different keys", func(t *testing.T) {
+		require.NoError(t, rdb.Del(ctx, "list1{t}").Err())
+		require.NoError(t, rdb.Del(ctx, "list2{t}").Err())
+		require.NoError(t, rdb.RPush(ctx, "list1{t}", "1").Err())
+		require.NoError(t, rdb.RPush(ctx, "list1{t}", "2").Err())
+		require.NoError(t, rdb.RPush(ctx, "list1{t}", "3").Err())
+		require.NoError(t, rdb.RPush(ctx, "list1{t}", "4").Err())
+		require.NoError(t, rdb.RPush(ctx, "list1{t}", "5").Err())
+		require.NoError(t, rdb.BLMove(ctx, "list1{t}", "list2{t}", "RIGHT", "LEFT", time.Millisecond*1000).Err())
+		require.NoError(t, rdb.BLMove(ctx, "list1{t}", "list2{t}", "LEFT", "RIGHT", time.Millisecond*1000).Err())
+		require.EqualValues(t, 3, rdb.LLen(ctx, "list1{t}").Val())
+		require.EqualValues(t, 2, rdb.LLen(ctx, "list2{t}").Val())
+		require.Equal(t, []string{"2", "3", "4"}, rdb.LRange(ctx, "list1{t}", 0, -1).Val())
+		require.Equal(t, []string{"5", "1"}, rdb.LRange(ctx, "list2{t}", 0, -1).Val())
+	})
+
+	for _, from := range []string{"LEFT", "RIGHT"} {
+		for _, to := range []string{"LEFT", "RIGHT"} {
+			t.Run(fmt.Sprintf("BLMOVE %s %s on the list node", from, to), func(t *testing.T) {
+				rd := srv.NewTCPClient()
+				defer func() { require.NoError(t, rd.Close()) }()
+				require.NoError(t, rdb.Del(ctx, "target_key{t}").Err())
+				require.NoError(t, rdb.RPush(ctx, "target_key{t}", 1).Err())
+				createList("list{t}", []string{"a", "b", "c", "d"})
+				require.NoError(t, rd.WriteArgs("blmove", "list{t}", "target_key{t}", from, to, "1"))
+				r, err1 := rd.ReadLine()
+				require.Equal(t, "$1", r)
+				require.NoError(t, err1)
+				elem, err2 := rd.ReadLine()
+				require.NoError(t, err2)
+				if from == "RIGHT" {
+					require.Equal(t, elem, "d")
+					require.Equal(t, []string{"a", "b", "c"}, rdb.LRange(ctx, "list{t}", 0, -1).Val())
+				} else {
+					require.Equal(t, elem, "a")
+					require.Equal(t, []string{"b", "c", "d"}, rdb.LRange(ctx, "list{t}", 0, -1).Val())
+				}
+				if to == "RIGHT" {
+					require.Equal(t, elem, rdb.RPop(ctx, "target_key{t}").Val())
+				} else {
+					require.Equal(t, elem, rdb.LPop(ctx, "target_key{t}").Val())
+				}
+			})
+		}
+	}
+
+	t.Run("Test BLMOVE block behaviour", func(t *testing.T) {
+		rd := srv.NewTCPClient()
+		defer func() { require.NoError(t, rd.Close()) }()
+		require.NoError(t, rdb.Del(ctx, "blist", "target").Err())
+		require.NoError(t, rd.WriteArgs("blmove", "blist", "target", "left", "right", "0"))
+		require.EqualValues(t, 2, rdb.LPush(ctx, "blist", "foo", "bar").Val())
+		rd.MustRead(t, "$3")
+		require.Equal(t, "bar", rdb.LRange(ctx, "target", 0, -1).Val()[0])
+	})
+
+	t.Run("LPOS rank negation overflow", func(t *testing.T) {
+		require.NoError(t, rdb.Del(ctx, "mylist").Err())
+		util.ErrorRegexp(t, rdb.Do(ctx, "LPOS", "mylist", "foo", "RANK", "-9223372036854775808").Err(), ".*rank would overflow.*")
+	})
+
+	for listType, large := range largeValue {
+		t.Run(fmt.Sprintf("LPOS basic usage - %s", listType), func(t *testing.T) {
+			createList("mylist", []string{"a", "b", "c", large, "2", "3", "c", "c"})
+			require.Equal(t, int64(0), rdb.LPos(ctx, "mylist", "a", redis.LPosArgs{}).Val())
+			require.Equal(t, int64(2), rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{}).Val())
+		})
+
+		t.Run("LPOS RANK option", func(t *testing.T) {
+			require.Equal(t, int64(2), rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{Rank: 1}).Val())
+			require.Equal(t, int64(6), rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{Rank: 2}).Val())
+			require.Error(t, rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{Rank: 4}).Err())
+			require.Equal(t, int64(7), rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{Rank: -1}).Val())
+			require.Equal(t, int64(6), rdb.LPos(ctx, "mylist", "c", redis.LPosArgs{Rank: -2}).Val())
+			err := rdb.Do(ctx, "LPOS", "mylist", "c", "RANK", 0).Err()
+			require.Error(t, err)
+			require.True(t, strings.Contains(err.Error(), "RANK can't be zero"))
+		})
+
+		t.Run("LPOS COUNT option", func(t *testing.T) {
+			require.Equal(t, []int64{2, 6, 7}, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{}).Val())
+			require.Equal(t, []int64{2}, rdb.LPosCount(ctx, "mylist", "c", 1, redis.LPosArgs{}).Val())
+			require.Equal(t, []int64{2, 6}, rdb.LPosCount(ctx, "mylist", "c", 2, redis.LPosArgs{}).Val())
+			require.Equal(t, []int64{2, 6, 7}, rdb.LPosCount(ctx, "mylist", "c", 100, redis.LPosArgs{}).Val())
+		})
+
+		t.Run("LPOS COUNT + RANK option", func(t *testing.T) {
+			require.Equal(t, []int64{6, 7}, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{Rank: 2}).Val())
+			require.Equal(t, []int64{7, 6}, rdb.LPosCount(ctx, "mylist", "c", 2, redis.LPosArgs{Rank: -1}).Val())
+		})
+
+		t.Run("LPOS non existing key", func(t *testing.T) {
+			require.Empty(t, rdb.LPosCount(ctx, "mylistxxx", "c", 0, redis.LPosArgs{Rank: 2}).Val())
+		})
+
+		t.Run("LPOS no match", func(t *testing.T) {
+			require.Empty(t, rdb.LPosCount(ctx, "mylist", "x", 2, redis.LPosArgs{Rank: -1}).Val())
+			require.Empty(t, rdb.LPos(ctx, "mylist", "x", redis.LPosArgs{Rank: -1}).Val())
+		})
+
+		t.Run("LPOS MAXLEN", func(t *testing.T) {
+			require.Equal(t, []int64{0}, rdb.LPosCount(ctx, "mylist", "a", 0, redis.LPosArgs{MaxLen: 1}).Val())
+			require.Empty(t, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{MaxLen: 1}).Val())
+			require.Equal(t, []int64{2}, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{MaxLen: 3}).Val())
+			require.Equal(t, []int64{7, 6}, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{MaxLen: 3, Rank: -1}).Val())
+			require.Equal(t, []int64{6}, rdb.LPosCount(ctx, "mylist", "c", 0, redis.LPosArgs{MaxLen: 7, Rank: 2}).Val())
+		})
+
+		t.Run("LPOS when RANK is greater than matches", func(t *testing.T) {
+			require.NoError(t, rdb.Del(ctx, "mylist").Err())
+			require.NoError(t, rdb.LPush(ctx, "mylist", "a").Err())
+			require.Empty(t, rdb.LPosCount(ctx, "mylist", "b", 10, redis.LPosArgs{Rank: 5}).Val())
+		})
+	}
+
+	for _, direction := range []string{"LEFT", "RIGHT"} {
+		key1 := "lmpop-list1"
+		key2 := "lmpop-list2"
+		rdb.Del(ctx, key1, key2)
+		require.EqualValues(t, 5, rdb.LPush(ctx, key1, "one", "two", "three", "four", "five").Val())
+		require.EqualValues(t, 5, rdb.LPush(ctx, key2, "ONE", "TWO", "THREE", "FOUR", "FIVE").Val())
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey countSingle %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 1, key1)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"five"}, resultVal)
+			} else {
+				require.Equal(t, []string{"one"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey countMulti %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 2, key1)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"four", "three"}, resultVal)
+			} else {
+				require.Equal(t, []string{"two", "three"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey countTooMuch %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 10, key1)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"two", "one"}, resultVal)
+			} else {
+				require.Equal(t, []string{"four", "five"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey empty %s", direction), func(t *testing.T) {
+			require.EqualError(t, rdb.LMPop(ctx, direction, 1, key1).Err(), redis.Nil.Error())
+		})
+
+		require.EqualValues(t, 2, rdb.LPush(ctx, key1, "six", "seven").Val())
+		t.Run(fmt.Sprintf("LMPOP test firstKey countOver %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 10, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"seven", "six"}, resultVal)
+			} else {
+				require.Equal(t, []string{"six", "seven"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test secondKey countSingle %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 1, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key2, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"FIVE"}, resultVal)
+			} else {
+				require.Equal(t, []string{"ONE"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test secondKey countMulti %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 2, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key2, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"FOUR", "THREE"}, resultVal)
+			} else {
+				require.Equal(t, []string{"TWO", "THREE"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test secondKey countOver %s", direction), func(t *testing.T) {
+			result := rdb.LMPop(ctx, direction, 10, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key2, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"TWO", "ONE"}, resultVal)
+			} else {
+				require.Equal(t, []string{"FOUR", "FIVE"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test bothKey empty %s", direction), func(t *testing.T) {
+			require.EqualError(t, rdb.LMPop(ctx, direction, 1, key1, key2).Err(), redis.Nil.Error())
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test dummyKey empty %s", direction), func(t *testing.T) {
+			require.EqualError(t, rdb.LMPop(ctx, direction, 1, "dummy1", "dummy2").Err(), redis.Nil.Error())
+		})
+
+		lmpopNoCount := func(c *redis.Client, ctx context.Context, direction string, keys ...string) *redis.KeyValuesCmd {
+			args := make([]interface{}, 2+len(keys), 5+len(keys))
+			args[0] = "lmpop"
+			args[1] = len(keys)
+			for i, key := range keys {
+				args[2+i] = key
+			}
+			args = append(args, strings.ToLower(direction))
+			cmd := redis.NewKeyValuesCmd(ctx, args...)
+			_ = c.Process(ctx, cmd)
+			return cmd
+		}
+		rdb.Del(ctx, key1, key2)
+		require.EqualValues(t, 2, rdb.LPush(ctx, key1, "one", "two").Val())
+		require.EqualValues(t, 2, rdb.LPush(ctx, key2, "ONE", "TWO").Val())
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey noCount one %s", direction), func(t *testing.T) {
+			result := lmpopNoCount(rdb, ctx, direction, key1)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"two"}, resultVal)
+			} else {
+				require.Equal(t, []string{"one"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test firstKey noCount one %s", direction), func(t *testing.T) {
+			result := lmpopNoCount(rdb, ctx, direction, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key1, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"one"}, resultVal)
+			} else {
+				require.Equal(t, []string{"two"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test oneKey noCount empty %s", direction), func(t *testing.T) {
+			require.EqualError(t, lmpopNoCount(rdb, ctx, direction, key1).Err(), redis.Nil.Error())
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test secondKey noCount one %s", direction), func(t *testing.T) {
+			result := lmpopNoCount(rdb, ctx, direction, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key2, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"TWO"}, resultVal)
+			} else {
+				require.Equal(t, []string{"ONE"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test secondKey noCount one %s", direction), func(t *testing.T) {
+			result := lmpopNoCount(rdb, ctx, direction, key1, key2)
+			resultKey, resultVal := result.Val()
+			require.NoError(t, result.Err())
+			require.EqualValues(t, key2, resultKey)
+			if direction == "LEFT" {
+				require.Equal(t, []string{"ONE"}, resultVal)
+			} else {
+				require.Equal(t, []string{"TWO"}, resultVal)
+			}
+		})
+
+		t.Run(fmt.Sprintf("LMPOP test bothKey noCount empty %s", direction), func(t *testing.T) {
+			require.EqualError(t, lmpopNoCount(rdb, ctx, direction, key1, key2).Err(), redis.Nil.Error())
+		})
+	}
 }
