@@ -120,10 +120,6 @@ def build(dir: str, jobs: Optional[int], ghproxy: bool, ninja: bool, unittest: b
     if D:
         cmake_options += [f"-D{o}" for o in D]
 
-    portable_flag_enabled = any("DPORTABLE" in o for o in cmake_options)
-    if not portable_flag_enabled:
-        cmake_options.append("-DPORTABLE=0")
-
     run(cmake, str(basedir), *cmake_options, verbose=True, cwd=dir)
 
     if skip_build:
@@ -266,9 +262,9 @@ def package_source(release_version: str, release_candidate_number: Optional[int]
     run(gpg, '--detach-sign', '--armor', tarball)
 
     # 4. Generate sha512 checksum
-    sha512sum = find_command('sha512sum', msg='sha512sum is required for source packaging')
+    shasum = find_command('shasum', msg='shasum is required for source packaging')
     with open(f'{tarball}.sha512', 'w+') as f:
-        run(sha512sum, tarball, stdout=f)
+        run(shasum, '-a', '512', tarball, stdout=f)
 
 
 def test_cpp(dir: str, rest: List[str]) -> None:
